--22-01-25[

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity AXI_LITE_source_v1_0_S00_AXI is
	generic (
		-- Users to add parameters here

		-- User parameters ends
		-- Do not modify the parameters beyond this line

		-- Width of S_AXI data bus
		C_S_AXI_DATA_WIDTH	: integer	:= 32;
		-- Width of S_AXI address bus
		C_S_AXI_ADDR_WIDTH	: integer	:= 8
	);
	port (
		-- Users to add ports here

		-- User ports ends
		-- Do not modify the ports beyond this line

		-- Global Clock Signal
		S_AXI_ACLK	: in std_logic;
		-- Global Reset Signal. This Signal is Active LOW
		S_AXI_ARESETN	: in std_logic;
		-- Write address (issued by master, acceped by Slave)
		S_AXI_AWADDR	: in std_logic_vector(C_S_AXI_ADDR_WIDTH-1 downto 0);
		-- Write channel Protection type. This signal indicates the
    		-- privilege and security level of the transaction, and whether
    		-- the transaction is a data access or an instruction access.
		S_AXI_AWPROT	: in std_logic_vector(2 downto 0);
		-- Write address valid. This signal indicates that the master signaling
    		-- valid write address and control information.
		S_AXI_AWVALID	: in std_logic;
		-- Write address ready. This signal indicates that the slave is ready
    		-- to accept an address and associated control signals.
		S_AXI_AWREADY	: out std_logic;
		-- Write data (issued by master, acceped by Slave) 
		S_AXI_WDATA	: in std_logic_vector(C_S_AXI_DATA_WIDTH-1 downto 0);
		-- Write strobes. This signal indicates which byte lanes hold
    		-- valid data. There is one write strobe bit for each eight
    		-- bits of the write data bus.    
		S_AXI_WSTRB	: in std_logic_vector((C_S_AXI_DATA_WIDTH/8)-1 downto 0);
		-- Write valid. This signal indicates that valid write
    		-- data and strobes are available.
		S_AXI_WVALID	: in std_logic;
		-- Write ready. This signal indicates that the slave
    		-- can accept the write data.
		S_AXI_WREADY	: out std_logic;
		-- Write response. This signal indicates the status
    		-- of the write transaction.
		S_AXI_BRESP	: out std_logic_vector(1 downto 0);
		-- Write response valid. This signal indicates that the channel
    		-- is signaling a valid write response.
		S_AXI_BVALID	: out std_logic;
		-- Response ready. This signal indicates that the master
    		-- can accept a write response.
		S_AXI_BREADY	: in std_logic;
		-- Read address (issued by master, acceped by Slave)
		S_AXI_ARADDR	: in std_logic_vector(C_S_AXI_ADDR_WIDTH-1 downto 0);
		-- Protection type. This signal indicates the privilege
    		-- and security level of the transaction, and whether the
    		-- transaction is a data access or an instruction access.
		S_AXI_ARPROT	: in std_logic_vector(2 downto 0);
		-- Read address valid. This signal indicates that the channel
    		-- is signaling valid read address and control information.
		S_AXI_ARVALID	: in std_logic;
		-- Read address ready. This signal indicates that the slave is
    		-- ready to accept an address and associated control signals.
		S_AXI_ARREADY	: out std_logic;
		-- Read data (issued by slave)
		S_AXI_RDATA	: out std_logic_vector(C_S_AXI_DATA_WIDTH-1 downto 0);
		-- Read response. This signal indicates the status of the
    		-- read transfer.
		S_AXI_RRESP	: out std_logic_vector(1 downto 0);
		-- Read valid. This signal indicates that the channel is
    		-- signaling the required read data.
		S_AXI_RVALID	: out std_logic;
		-- Read ready. This signal indicates that the master can
    		-- accept the read data and response information.
		S_AXI_RREADY	: in std_logic
	);
end AXI_LITE_source_v1_0_S00_AXI;

architecture arch_imp of AXI_LITE_source_v1_0_S00_AXI is

	-- AXI4LITE signals
	signal axi_awaddr	: std_logic_vector(C_S_AXI_ADDR_WIDTH-1 downto 0);
	signal axi_awready	: std_logic;
	signal axi_wready	: std_logic;
	signal axi_bresp	: std_logic_vector(1 downto 0);
	signal axi_bvalid	: std_logic;
	signal axi_araddr	: std_logic_vector(C_S_AXI_ADDR_WIDTH-1 downto 0);
	signal axi_arready	: std_logic;
	signal axi_rdata	: std_logic_vector(C_S_AXI_DATA_WIDTH-1 downto 0);
	signal axi_rresp	: std_logic_vector(1 downto 0);
	signal axi_rvalid	: std_logic;

	-- Example-specific design signals
	-- local parameter for addressing 32 bit / 64 bit C_S_AXI_DATA_WIDTH
	-- ADDR_LSB is used for addressing 32/64 bit registers/memories
	-- ADDR_LSB = 2 for 32 bits (n downto 2)
	-- ADDR_LSB = 3 for 64 bits (n downto 3)
	constant ADDR_LSB  : integer := (C_S_AXI_DATA_WIDTH/32)+ 1;
	constant OPT_MEM_ADDR_BITS : integer := 5;

    
        component main_sub is
        Port (
    --input--
        clk : in STD_LOGIC;
        reset : in std_logic;
        sig_data_in : in std_logic;
        cube_data : in unsigned(99 downto 0);
    --output--
        sig_solve_end : out std_logic;--1FvZI
        sig_solve_step : out unsigned(3 downto 0) 
            );
        end component;
        
 

	---- Number of Slave Registers 4
	signal slv_reg0	:std_logic_vector(C_S_AXI_DATA_WIDTH-1 downto 0);
	signal slv_reg1	:std_logic_vector(C_S_AXI_DATA_WIDTH-1 downto 0);
	signal slv_reg2	:std_logic_vector(C_S_AXI_DATA_WIDTH-1 downto 0);
	signal slv_reg3	:std_logic_vector(C_S_AXI_DATA_WIDTH-1 downto 0);
	signal slv_reg4	:std_logic_vector(C_S_AXI_DATA_WIDTH-1 downto 0);
	signal slv_reg5	:std_logic_vector(C_S_AXI_DATA_WIDTH-1 downto 0);
	signal slv_reg6	:std_logic_vector(C_S_AXI_DATA_WIDTH-1 downto 0);
	signal slv_reg7	:std_logic_vector(C_S_AXI_DATA_WIDTH-1 downto 0);
	signal slv_reg8	:std_logic_vector(C_S_AXI_DATA_WIDTH-1 downto 0);
	signal slv_reg9	:std_logic_vector(C_S_AXI_DATA_WIDTH-1 downto 0);
	signal slv_reg10	:std_logic_vector(C_S_AXI_DATA_WIDTH-1 downto 0);
	signal slv_reg11	:std_logic_vector(C_S_AXI_DATA_WIDTH-1 downto 0);
	signal slv_reg12	:std_logic_vector(C_S_AXI_DATA_WIDTH-1 downto 0);
	signal slv_reg13	:std_logic_vector(C_S_AXI_DATA_WIDTH-1 downto 0);
	signal slv_reg14	:std_logic_vector(C_S_AXI_DATA_WIDTH-1 downto 0);
	signal slv_reg15	:std_logic_vector(C_S_AXI_DATA_WIDTH-1 downto 0);
	signal slv_reg16	:std_logic_vector(C_S_AXI_DATA_WIDTH-1 downto 0);
	signal slv_reg17	:std_logic_vector(C_S_AXI_DATA_WIDTH-1 downto 0);
	signal slv_reg18	:std_logic_vector(C_S_AXI_DATA_WIDTH-1 downto 0);
	signal slv_reg19	:std_logic_vector(C_S_AXI_DATA_WIDTH-1 downto 0);
	signal slv_reg20	:std_logic_vector(C_S_AXI_DATA_WIDTH-1 downto 0);
	signal slv_reg21	:std_logic_vector(C_S_AXI_DATA_WIDTH-1 downto 0);
	signal slv_reg22	:std_logic_vector(C_S_AXI_DATA_WIDTH-1 downto 0);
	signal slv_reg23	:std_logic_vector(C_S_AXI_DATA_WIDTH-1 downto 0);
	signal slv_reg24	:std_logic_vector(C_S_AXI_DATA_WIDTH-1 downto 0);
	signal slv_reg25	:std_logic_vector(C_S_AXI_DATA_WIDTH-1 downto 0);
	signal slv_reg26	:std_logic_vector(C_S_AXI_DATA_WIDTH-1 downto 0);
	signal slv_reg27	:std_logic_vector(C_S_AXI_DATA_WIDTH-1 downto 0);
	signal slv_reg28	:std_logic_vector(C_S_AXI_DATA_WIDTH-1 downto 0);
	signal slv_reg29	:std_logic_vector(C_S_AXI_DATA_WIDTH-1 downto 0);
	signal slv_reg30	:std_logic_vector(C_S_AXI_DATA_WIDTH-1 downto 0);
	signal slv_reg31	:std_logic_vector(C_S_AXI_DATA_WIDTH-1 downto 0);
	signal slv_reg32	:std_logic_vector(C_S_AXI_DATA_WIDTH-1 downto 0);
	signal slv_reg33	:std_logic_vector(C_S_AXI_DATA_WIDTH-1 downto 0);
	signal slv_reg34	:std_logic_vector(C_S_AXI_DATA_WIDTH-1 downto 0);
	signal slv_reg35	:std_logic_vector(C_S_AXI_DATA_WIDTH-1 downto 0);
	signal slv_reg36	:std_logic_vector(C_S_AXI_DATA_WIDTH-1 downto 0);
	signal slv_reg37	:std_logic_vector(C_S_AXI_DATA_WIDTH-1 downto 0);
	signal slv_reg38	:std_logic_vector(C_S_AXI_DATA_WIDTH-1 downto 0);
	signal slv_reg39	:std_logic_vector(C_S_AXI_DATA_WIDTH-1 downto 0);
	signal slv_reg40	:std_logic_vector(C_S_AXI_DATA_WIDTH-1 downto 0);
	signal slv_reg41	:std_logic_vector(C_S_AXI_DATA_WIDTH-1 downto 0);
	signal slv_reg42	:std_logic_vector(C_S_AXI_DATA_WIDTH-1 downto 0);
	signal slv_reg43	:std_logic_vector(C_S_AXI_DATA_WIDTH-1 downto 0);
	signal slv_reg44	:std_logic_vector(C_S_AXI_DATA_WIDTH-1 downto 0);
	signal slv_reg45	:std_logic_vector(C_S_AXI_DATA_WIDTH-1 downto 0);
	signal slv_reg46	:std_logic_vector(C_S_AXI_DATA_WIDTH-1 downto 0);
	signal slv_reg47	:std_logic_vector(C_S_AXI_DATA_WIDTH-1 downto 0);
	signal slv_reg48	:std_logic_vector(C_S_AXI_DATA_WIDTH-1 downto 0);
	signal slv_reg49	:std_logic_vector(C_S_AXI_DATA_WIDTH-1 downto 0);
	signal slv_reg50	:std_logic_vector(C_S_AXI_DATA_WIDTH-1 downto 0);
	signal slv_reg51	:std_logic_vector(C_S_AXI_DATA_WIDTH-1 downto 0);
	signal slv_reg52	:std_logic_vector(C_S_AXI_DATA_WIDTH-1 downto 0);
	signal slv_reg53	:std_logic_vector(C_S_AXI_DATA_WIDTH-1 downto 0);
	signal slv_reg54	:std_logic_vector(C_S_AXI_DATA_WIDTH-1 downto 0);
	signal slv_reg_rden	: std_logic;
	signal slv_reg_wren	: std_logic;
	signal reg_data_out	:std_logic_vector(C_S_AXI_DATA_WIDTH-1 downto 0);
	signal byte_index	: integer;
	signal aw_en	: std_logic;
    signal rets : std_logic := '0';
    signal rets_delay : std_logic := '0';--2clocko͗p
    signal in_data : unsigned(99 downto 0) := (others => '0');
    signal fdata0 : unsigned(19 downto 0) := (others => '0');
    signal fdata1 : unsigned(19 downto 0) := (others => '0');
    signal fdata2 : unsigned(29 downto 0) := (others => '0');
    signal fdata3 : unsigned(29 downto 0) := (others => '0');
    signal sig_data_in : std_logic := '0'; 
    signal sig_solve_step : unsigned(3 downto 0) := (others => '0');
    signal sig_solve_end : std_logic := '0';
    signal step_addr: unsigned(5 downto 0):= (others => '0');
    signal solve_step_delay: unsigned(3 downto 0) := (others => '0');
    type REOUT is array (0 to  30 - 1) of std_logic_vector(32 - 1 downto 0);
    signal regdata : REOUT := (others => (others => '0'));
    signal retscount : unsigned(1 downto 0):="00";
    signal sig_store_step : std_logic := '0';
    signal clk_count: integer  := 0;
    signal CLOK : std_logic:='0';
    signal cube_data_temp : unsigned(99 downto 0);
    signal addr_count : integer := 0;
    signal clk_expand_count_r : integer;
    signal clk_expand_count_rd : integer;
    signal clk_expand_count_ss : unsigned(2 downto 0) := (others => '0');
    signal clk_expand : integer :=1; --0:1/2,1:1/4,2:1/6,3:1/8clk=>clok
begin
	-- I/O Connections assignments

	S_AXI_AWREADY	<= axi_awready;
	S_AXI_WREADY	<= axi_wready;
	S_AXI_BRESP	<= axi_bresp;
	S_AXI_BVALID	<= axi_bvalid;
	S_AXI_ARREADY	<= axi_arready;
	S_AXI_RDATA	<= axi_rdata;
	S_AXI_RRESP	<= axi_rresp;
	S_AXI_RVALID	<= axi_rvalid;
	-- Implement axi_awready generation
	-- axi_awready is asserted for one S_AXI_ACLK clock cycle when both
	-- S_AXI_AWVALID and S_AXI_WVALID are asserted. axi_awready is
	-- de-asserted when reset is low.
    process (slv_reg51,slv_reg52,slv_reg53,slv_reg54)
	begin
	       fdata0<= unsigned(slv_reg51(19 downto 0));
	       fdata1<= unsigned(slv_reg52(19 downto 0));
	       fdata2<= unsigned(slv_reg53(29 downto 0));
	       fdata3<= unsigned(slv_reg54(29 downto 0));
	end process;
	process (fdata0 , fdata1 , fdata2 , fdata3)
	begin
	 
	        
	       in_data<=fdata0 & fdata1 & fdata2 & fdata3;
	end process;
	
	process(S_AXI_ACLK)
    begin
        if rising_edge(S_AXI_ACLK) then
            if(slv_reg_wren = '1') then
                    retscount<=retscount + to_unsigned(1,retscount'length);
            end if;
        end if;
    
    end process;
    
    --gen rets:timing of cube_data<=in_data--
    process(S_AXI_ACLK)
    begin
        if rising_edge(S_AXI_ACLK) then
            if(to_integer(retscount)=3) then
                if(slv_reg_wren  = '1') then
                    rets<='1';
                    clk_expand_count_r<=0;
                end if;
            else
                clk_expand_count_r<=clk_expand_count_r+1;
                clk_expand_count_rd<=clk_expand_count_rd+1;
                if clk_expand_count_rd=clk_expand then
                    rets_delay<='0';
                    clk_expand_count_rd<=0;
                elsif clk_expand_count_r=clk_expand then
                    rets<='0';
                    rets_delay<='1';
                    clk_expand_count_r<=0;
                    clk_expand_count_rd<=0;
                end if;
            end if;
        end if;        
    end process;
    
    --sig_store_step:store step_data at 1 clock--
    process(S_AXI_ACLK)
    begin
        if rising_edge(S_AXI_ACLK) then
            if (sig_store_step = '1') then
                sig_store_step <= '0';
            elsif (to_integer(sig_solve_step) /= 0 and to_integer(clk_expand_count_ss) = 0) then
                    sig_store_step <= '1';
            end if;
        end if;
    end process;
    
    --controll sig_store_step--
    process(S_AXI_ACLK)
    begin
        if (rising_edge(S_AXI_ACLK)) then
            if (to_integer(sig_solve_step) /= 0) then
                clk_expand_count_ss <= clk_expand_count_ss + to_unsigned(1, clk_expand_count_ss'length);
            else
                clk_expand_count_ss <= (others => '0');
            end if;
        end if;
    end process;
    
    --input cube_data timing--
    process(S_AXI_ACLK)
    begin
        if rising_edge(S_AXI_ACLK) then
            if (rets='1' or rets_delay = '1') then
                sig_data_in<='1';
                cube_data_temp<=in_data;
            elsif (sig_data_in='1') then
                sig_data_in<='0';
                cube_data_temp<=( others => '0');
            end if;
        end if;    
    end process;
    
	process (S_AXI_ACLK)
	begin
	  if rising_edge(S_AXI_ACLK) then 
	    if S_AXI_ARESETN = '0' then
	      axi_awready <= '0';
	      aw_en <= '1';
	    else
	      if (axi_awready = '0' and S_AXI_AWVALID = '1' and S_AXI_WVALID = '1' and aw_en = '1') then
	        -- slave is ready to accept write address when
	        -- there is a valid write address and write data
	        -- on the write address and data bus. This design 
	        -- expects no outstanding transactions. 
	           axi_awready <= '1';
	           aw_en <= '0';
	        elsif (S_AXI_BREADY = '1' and axi_bvalid = '1') then
	           aw_en <= '1';
	           axi_awready <= '0';
	      else
	        axi_awready <= '0';
	      end if;
	    end if;
	  end if;
	end process;

	-- Implement axi_awaddr latching
	-- This process is used to latch the address when both 
	-- S_AXI_AWVALID and S_AXI_WVALID are valid. 

	process (S_AXI_ACLK)
	begin
        if rising_edge(S_AXI_ACLK) then 
            if S_AXI_ARESETN = '0' then
                
                axi_awaddr <= (others => '0');
                addr_count<=0;
            else
                if (addr_count<4) then
                    if (axi_awready = '0' and S_AXI_AWVALID = '1' and S_AXI_WVALID = '1' and aw_en = '1') then
                    -- Write Address latching
                        axi_awaddr <= S_AXI_AWADDR;
                        addr_count<=addr_count+1;
                    end if;
                else
                    if (to_integer(sig_solve_step) /= 0) then
                        if (to_integer(step_addr) = 50) then
                            if (rets = '1') then
                                step_addr <= (others => '0');--"100010";34
                            else
                                step_addr <= step_addr;
                            end if;
                        elsif (sig_store_step = '1') then
                            step_addr <= step_addr+to_unsigned(1,step_addr'length);
                        end if;
                    end if;
                    axi_awaddr <= std_logic_vector(step_addr) & "00" ;
                    --regdata(to_integer(step_addr))<=std_logic_vector(to_unsigned(to_integer(sig_solve_step), C_S_AXI_DATA_WIDTH));
                end if;
            end if;
        end if;
	end process;
	
	process (S_AXI_ACLK)
	begin
	   if rising_edge(S_AXI_ACLK) then
	       if (to_integer(clk_expand_count_ss) < 2) then
    	       solve_step_delay <= sig_solve_step;
    	   else
    	       solve_step_delay <= (others => '0');
    	   end if;
	   end if;
	end process;
	-- Implement axi_wready generation
	-- axi_wready is asserted for one S_AXI_ACLK clock cycle when both
	-- S_AXI_AWVALID and S_AXI_WVALID are asserted. axi_wready is 
	-- de-asserted when reset is low. 

	process (S_AXI_ACLK)
	begin
	  if rising_edge(S_AXI_ACLK) then 
	    if S_AXI_ARESETN = '0' then
	      axi_wready <= '0';
	    else
	      if (axi_wready = '0' and S_AXI_WVALID = '1' and S_AXI_AWVALID = '1' and aw_en = '1') then
	          -- slave is ready to accept write data when 
	          -- there is a valid write address and write data
	          -- on the write address and data bus. This design 
	          -- expects no outstanding transactions.           
	          axi_wready <= '1';
	      else
	        axi_wready <= '0';
	      end if;
	    end if;
	  end if;
	end process; 

	-- Implement memory mapped register select and write logic generation
	-- The write data is accepted and written to memory mapped registers when
	-- axi_awready, S_AXI_WVALID, axi_wready and S_AXI_WVALID are asserted. Write strobes are used to
	-- select byte enables of slave registers while writing.
	-- These registers are cleared when reset (active low) is applied.
	-- Slave register write enable is asserted when valid address and data are available
	-- and the slave is ready to accept the write address and write data.
	
	--݋M
	slv_reg_wren <= axi_wready and S_AXI_WVALID and axi_awready and S_AXI_AWVALID ;

	process (S_AXI_ACLK)
	variable loc_addr :std_logic_vector(OPT_MEM_ADDR_BITS downto 0); 
	begin
	  if rising_edge(S_AXI_ACLK) then 
	    if S_AXI_ARESETN = '0' then
	      slv_reg0 <= (others => '0');
	      slv_reg1 <= (others => '0');
	      slv_reg2 <= (others => '0');
	      slv_reg3 <= (others => '0');
	      slv_reg4 <= (others => '0');
	      slv_reg5 <= (others => '0');
	      slv_reg6 <= (others => '0');
	      slv_reg7 <= (others => '0');
	      slv_reg8 <= (others => '0');
	      slv_reg9 <= (others => '0');
	      slv_reg10 <= (others => '0');
	      slv_reg11 <= (others => '0');
	      slv_reg12 <= (others => '0');
	      slv_reg13 <= (others => '0');
	      slv_reg14 <= (others => '0');
	      slv_reg15 <= (others => '0');
	      slv_reg16 <= (others => '0');
	      slv_reg17 <= (others => '0');
	      slv_reg18 <= (others => '0');
	      slv_reg19 <= (others => '0');
	      slv_reg20 <= (others => '0');
	      slv_reg21 <= (others => '0');
	      slv_reg22 <= (others => '0');
	      slv_reg23 <= (others => '0');
	      slv_reg24 <= (others => '0');
	      slv_reg25 <= (others => '0');
	      slv_reg26 <= (others => '0');
	      slv_reg27 <= (others => '0');
	      slv_reg28 <= (others => '0');
	      slv_reg29 <= (others => '0');
	      slv_reg30 <= (others => '0');
	      slv_reg31 <= (others => '0');
	      slv_reg32 <= (others => '0');
	      slv_reg33 <= (others => '0');
	      slv_reg34 <= (others => '0');
	      slv_reg35 <= (others => '0');
	      slv_reg36 <= (others => '0');
	      slv_reg37 <= (others => '0');
	      slv_reg38 <= (others => '0');
	      slv_reg39 <= (others => '0');
	      slv_reg40 <= (others => '0');
	      slv_reg41 <= (others => '0');
	      slv_reg42 <= (others => '0');
	      slv_reg43 <= (others => '0');
	      slv_reg44 <= (others => '0');
	      slv_reg45 <= (others => '0');
	      slv_reg46 <= (others => '0');
	      slv_reg47 <= (others => '0');
	      slv_reg48 <= (others => '0');
	      slv_reg49 <= (others => '0');
	      slv_reg50 <= (others => '0');
	      slv_reg51 <= (others => '0');
	      slv_reg52 <= (others => '0');
	      slv_reg53 <= (others => '0');
	      slv_reg54 <= (others => '0');
	    else
	      loc_addr := axi_awaddr(ADDR_LSB + OPT_MEM_ADDR_BITS downto ADDR_LSB);
	      if (slv_reg_wren = '1') then
	        case loc_addr is
	          when b"110011" =>
	            for byte_index in 0 to (C_S_AXI_DATA_WIDTH/8-1) loop
	              if ( S_AXI_WSTRB(byte_index) = '1' ) then
	                -- Respective byte enables are asserted as per write strobes                   
	                -- slave registor 51
	                slv_reg51(byte_index*8+7 downto byte_index*8) <= S_AXI_WDATA(byte_index*8+7 downto byte_index*8);
	              end if;
	            end loop;
	          when b"110100" =>
	            for byte_index in 0 to (C_S_AXI_DATA_WIDTH/8-1) loop
	              if ( S_AXI_WSTRB(byte_index) = '1' ) then
	                -- Respective byte enables are asserted as per write strobes                   
	                -- slave registor 52
	                slv_reg52(byte_index*8+7 downto byte_index*8) <= S_AXI_WDATA(byte_index*8+7 downto byte_index*8);
	              end if;
	            end loop;
	          when b"110101" =>
	            for byte_index in 0 to (C_S_AXI_DATA_WIDTH/8-1) loop
	              if ( S_AXI_WSTRB(byte_index) = '1' ) then
	                -- Respective byte enables are asserted as per write strobes                   
	                -- slave registor 53
	                slv_reg53(byte_index*8+7 downto byte_index*8) <= S_AXI_WDATA(byte_index*8+7 downto byte_index*8);
	              end if;
	            end loop;
	          when b"110110" =>
	            for byte_index in 0 to (C_S_AXI_DATA_WIDTH/8-1) loop
	              if ( S_AXI_WSTRB(byte_index) = '1' ) then
	                -- Respective byte enables are asserted as per write strobes                   
	                -- slave registor 54
	                slv_reg54(byte_index*8+7 downto byte_index*8) <= S_AXI_WDATA(byte_index*8+7 downto byte_index*8);
	              end if;
	            end loop;
	          when others =>
	            slv_reg0 <= slv_reg0;
	            slv_reg1 <= slv_reg1;
	            slv_reg2 <= slv_reg2;
	            slv_reg3 <= slv_reg3;
	            slv_reg4 <= slv_reg4;
	            slv_reg5 <= slv_reg5;
	            slv_reg6 <= slv_reg6;
	            slv_reg7 <= slv_reg7;
	            slv_reg8 <= slv_reg8;
	            slv_reg9 <= slv_reg9;
	            slv_reg10 <= slv_reg10;
	            slv_reg11 <= slv_reg11;
	            slv_reg12 <= slv_reg12;
	            slv_reg13 <= slv_reg13;
	            slv_reg14 <= slv_reg14;
	            slv_reg15 <= slv_reg15;
	            slv_reg16 <= slv_reg16;
	            slv_reg17 <= slv_reg17;
	            slv_reg18 <= slv_reg18;
	            slv_reg19 <= slv_reg19;
	            slv_reg20 <= slv_reg20;
	            slv_reg21 <= slv_reg21;
	            slv_reg22 <= slv_reg22;
	            slv_reg23 <= slv_reg23;
	            slv_reg24 <= slv_reg24;
	            slv_reg25 <= slv_reg25;
	            slv_reg26 <= slv_reg26;
	            slv_reg27 <= slv_reg27;
	            slv_reg28 <= slv_reg28;
	            slv_reg29 <= slv_reg29;
	            slv_reg30 <= slv_reg30;
	            slv_reg31 <= slv_reg31;
	            slv_reg32 <= slv_reg32;
	            slv_reg33 <= slv_reg33;
	            slv_reg34 <= slv_reg34;
	            slv_reg35 <= slv_reg35;
	            slv_reg36 <= slv_reg36;
	            slv_reg37 <= slv_reg37;
	            slv_reg38 <= slv_reg38;
	            slv_reg39 <= slv_reg39;
	            slv_reg40 <= slv_reg40;
	            slv_reg41 <= slv_reg41;
	            slv_reg42 <= slv_reg42;
	            slv_reg43 <= slv_reg43;
	            slv_reg44 <= slv_reg44;
	            slv_reg45 <= slv_reg45;
	            slv_reg46 <= slv_reg46;
	            slv_reg47 <= slv_reg47;
	            slv_reg48 <= slv_reg48;
	            slv_reg49 <= slv_reg49;
	            slv_reg50 <= slv_reg50;
	            slv_reg51 <= slv_reg51;
	            slv_reg52 <= slv_reg52;
	            slv_reg53 <= slv_reg53;
	            slv_reg54 <= slv_reg54;
	          end case;
          elsif (to_integer(step_addr) /= 0) then
	          case loc_addr is
                   when b"000000"=> slv_reg0 <= std_logic_vector(to_unsigned(0, slv_reg0'length-sig_solve_step'length)) & std_logic_vector(solve_step_delay);
                   when b"000001"=> slv_reg1 <= std_logic_vector(to_unsigned(0, slv_reg0'length-sig_solve_step'length)) & std_logic_vector(solve_step_delay);
                   when b"000010"=> slv_reg2 <= std_logic_vector(to_unsigned(0, slv_reg0'length-sig_solve_step'length)) & std_logic_vector(solve_step_delay);
                   when b"000011"=> slv_reg3 <= std_logic_vector(to_unsigned(0, slv_reg0'length-sig_solve_step'length)) & std_logic_vector(solve_step_delay);
                   when b"000100"=> slv_reg4 <= std_logic_vector(to_unsigned(0, slv_reg0'length-sig_solve_step'length)) & std_logic_vector(solve_step_delay);
                   when b"000101"=> slv_reg5 <= std_logic_vector(to_unsigned(0, slv_reg0'length-sig_solve_step'length)) & std_logic_vector(solve_step_delay);
                   when b"000110"=> slv_reg6 <= std_logic_vector(to_unsigned(0, slv_reg0'length-sig_solve_step'length)) & std_logic_vector(solve_step_delay);
                   when b"000111"=> slv_reg7 <= std_logic_vector(to_unsigned(0, slv_reg0'length-sig_solve_step'length)) & std_logic_vector(solve_step_delay);
                   when b"001000"=> slv_reg8 <= std_logic_vector(to_unsigned(0, slv_reg0'length-sig_solve_step'length)) & std_logic_vector(solve_step_delay);
                   when b"001001"=> slv_reg9 <= std_logic_vector(to_unsigned(0, slv_reg0'length-sig_solve_step'length)) & std_logic_vector(solve_step_delay);
                   when b"001010"=> slv_reg10 <= std_logic_vector(to_unsigned(0, slv_reg0'length-sig_solve_step'length)) & std_logic_vector(solve_step_delay);
                   when b"001011"=> slv_reg11 <= std_logic_vector(to_unsigned(0, slv_reg0'length-sig_solve_step'length)) & std_logic_vector(solve_step_delay);
                   when b"001100"=> slv_reg12 <= std_logic_vector(to_unsigned(0, slv_reg0'length-sig_solve_step'length)) & std_logic_vector(solve_step_delay);
                   when b"001101"=> slv_reg13 <= std_logic_vector(to_unsigned(0, slv_reg0'length-sig_solve_step'length)) & std_logic_vector(solve_step_delay);
                   when b"001110"=> slv_reg14 <= std_logic_vector(to_unsigned(0, slv_reg0'length-sig_solve_step'length)) & std_logic_vector(solve_step_delay);
                   when b"001111"=> slv_reg15 <= std_logic_vector(to_unsigned(0, slv_reg0'length-sig_solve_step'length)) & std_logic_vector(solve_step_delay);
                   when b"010000"=> slv_reg16 <= std_logic_vector(to_unsigned(0, slv_reg0'length-sig_solve_step'length)) & std_logic_vector(solve_step_delay);
                   when b"010001"=> slv_reg17 <= std_logic_vector(to_unsigned(0, slv_reg0'length-sig_solve_step'length)) & std_logic_vector(solve_step_delay);
                   when b"010010"=> slv_reg18 <= std_logic_vector(to_unsigned(0, slv_reg0'length-sig_solve_step'length)) & std_logic_vector(solve_step_delay);
                   when b"010011"=> slv_reg19 <= std_logic_vector(to_unsigned(0, slv_reg0'length-sig_solve_step'length)) & std_logic_vector(solve_step_delay);
                   when b"010100"=> slv_reg20 <= std_logic_vector(to_unsigned(0, slv_reg0'length-sig_solve_step'length)) & std_logic_vector(solve_step_delay);
                   when b"010101"=> slv_reg21 <= std_logic_vector(to_unsigned(0, slv_reg0'length-sig_solve_step'length)) & std_logic_vector(solve_step_delay);
                   when b"010110"=> slv_reg22 <= std_logic_vector(to_unsigned(0, slv_reg0'length-sig_solve_step'length)) & std_logic_vector(solve_step_delay);
                   when b"010111"=> slv_reg23 <= std_logic_vector(to_unsigned(0, slv_reg0'length-sig_solve_step'length)) & std_logic_vector(solve_step_delay);
                   when b"011000"=> slv_reg24 <= std_logic_vector(to_unsigned(0, slv_reg0'length-sig_solve_step'length)) & std_logic_vector(solve_step_delay);
                   when b"011001"=> slv_reg25 <= std_logic_vector(to_unsigned(0, slv_reg0'length-sig_solve_step'length)) & std_logic_vector(solve_step_delay);
                   when b"011010"=> slv_reg26 <= std_logic_vector(to_unsigned(0, slv_reg0'length-sig_solve_step'length)) & std_logic_vector(solve_step_delay);
                   when b"011011"=> slv_reg27 <= std_logic_vector(to_unsigned(0, slv_reg0'length-sig_solve_step'length)) & std_logic_vector(solve_step_delay);
                   when b"011100"=> slv_reg28 <= std_logic_vector(to_unsigned(0, slv_reg0'length-sig_solve_step'length)) & std_logic_vector(solve_step_delay);
                   when b"011101"=> slv_reg29 <= std_logic_vector(to_unsigned(0, slv_reg0'length-sig_solve_step'length)) & std_logic_vector(solve_step_delay);
                   when b"011110"=> slv_reg30 <= std_logic_vector(to_unsigned(0, slv_reg0'length-sig_solve_step'length)) & std_logic_vector(solve_step_delay);
                   when b"011111"=> slv_reg31 <= std_logic_vector(to_unsigned(0, slv_reg0'length-sig_solve_step'length)) & std_logic_vector(solve_step_delay);
                   when b"100000"=> slv_reg32 <= std_logic_vector(to_unsigned(0, slv_reg0'length-sig_solve_step'length)) & std_logic_vector(solve_step_delay);
                   when b"100001"=> slv_reg33 <= std_logic_vector(to_unsigned(0, slv_reg0'length-sig_solve_step'length)) & std_logic_vector(solve_step_delay);
                   when b"100010"=> slv_reg34 <= std_logic_vector(to_unsigned(0, slv_reg0'length-sig_solve_step'length)) & std_logic_vector(solve_step_delay);
                   when b"100011"=> slv_reg35 <= std_logic_vector(to_unsigned(0, slv_reg0'length-sig_solve_step'length)) & std_logic_vector(solve_step_delay);
                   when b"100100"=> slv_reg36 <= std_logic_vector(to_unsigned(0, slv_reg0'length-sig_solve_step'length)) & std_logic_vector(solve_step_delay);
                   when b"100101"=> slv_reg37 <= std_logic_vector(to_unsigned(0, slv_reg0'length-sig_solve_step'length)) & std_logic_vector(solve_step_delay);
                   when b"100110"=> slv_reg38 <= std_logic_vector(to_unsigned(0, slv_reg0'length-sig_solve_step'length)) & std_logic_vector(solve_step_delay);
                   when b"100111"=> slv_reg39 <= std_logic_vector(to_unsigned(0, slv_reg0'length-sig_solve_step'length)) & std_logic_vector(solve_step_delay);
                   when b"101000"=> slv_reg40 <= std_logic_vector(to_unsigned(0, slv_reg0'length-sig_solve_step'length)) & std_logic_vector(solve_step_delay);
                   when b"101001"=> slv_reg41 <= std_logic_vector(to_unsigned(0, slv_reg0'length-sig_solve_step'length)) & std_logic_vector(solve_step_delay);
                   when b"101010"=> slv_reg42 <= std_logic_vector(to_unsigned(0, slv_reg0'length-sig_solve_step'length)) & std_logic_vector(solve_step_delay);
                   when b"101011"=> slv_reg43 <= std_logic_vector(to_unsigned(0, slv_reg0'length-sig_solve_step'length)) & std_logic_vector(solve_step_delay);
                   when b"101100"=> slv_reg44 <= std_logic_vector(to_unsigned(0, slv_reg0'length-sig_solve_step'length)) & std_logic_vector(solve_step_delay);
                   when b"101101"=> slv_reg45 <= std_logic_vector(to_unsigned(0, slv_reg0'length-sig_solve_step'length)) & std_logic_vector(solve_step_delay);
                   when b"101110"=> slv_reg46 <= std_logic_vector(to_unsigned(0, slv_reg0'length-sig_solve_step'length)) & std_logic_vector(solve_step_delay);
                   when b"101111"=> slv_reg47 <= std_logic_vector(to_unsigned(0, slv_reg0'length-sig_solve_step'length)) & std_logic_vector(solve_step_delay);
                   when b"110000"=> slv_reg48 <= std_logic_vector(to_unsigned(0, slv_reg0'length-sig_solve_step'length)) & std_logic_vector(solve_step_delay);
                   when b"110001"=> slv_reg49 <= std_logic_vector(to_unsigned(0, slv_reg0'length-sig_solve_step'length)) & std_logic_vector(solve_step_delay);
                   when b"110010"=> slv_reg50 <= std_logic_vector(to_unsigned(0, slv_reg0'length-sig_solve_step'length)) & std_logic_vector(solve_step_delay);
                   when b"110011"=> slv_reg51 <= std_logic_vector(to_unsigned(0, slv_reg0'length-sig_solve_step'length)) & std_logic_vector(solve_step_delay);
                   when b"110100"=> slv_reg52 <= std_logic_vector(to_unsigned(0, slv_reg0'length-sig_solve_step'length)) & std_logic_vector(solve_step_delay);
                   when b"110101"=> slv_reg53 <= std_logic_vector(to_unsigned(0, slv_reg0'length-sig_solve_step'length)) & std_logic_vector(solve_step_delay);
                   when b"110110"=> slv_reg54 <= std_logic_vector(to_unsigned(0, slv_reg0'length-sig_solve_step'length)) & std_logic_vector(solve_step_delay);


	          when others =>
	            slv_reg0 <= slv_reg0;
	            slv_reg1 <= slv_reg1;
	            slv_reg2 <= slv_reg2;
	            slv_reg3 <= slv_reg3;
	            slv_reg4 <= slv_reg4;
	            slv_reg5 <= slv_reg5;
	            slv_reg6 <= slv_reg6;
	            slv_reg7 <= slv_reg7;
	            slv_reg8 <= slv_reg8;
	            slv_reg9 <= slv_reg9;
	            slv_reg10 <= slv_reg10;
	            slv_reg11 <= slv_reg11;
	            slv_reg12 <= slv_reg12;
	            slv_reg13 <= slv_reg13;
	            slv_reg14 <= slv_reg14;
	            slv_reg15 <= slv_reg15;
	            slv_reg16 <= slv_reg16;
	            slv_reg17 <= slv_reg17;
	            slv_reg18 <= slv_reg18;
	            slv_reg19 <= slv_reg19;
	            slv_reg20 <= slv_reg20;
	            slv_reg21 <= slv_reg21;
	            slv_reg22 <= slv_reg22;
	            slv_reg23 <= slv_reg23;
	            slv_reg24 <= slv_reg24;
	            slv_reg25 <= slv_reg25;
	            slv_reg26 <= slv_reg26;
	            slv_reg27 <= slv_reg27;
	            slv_reg28 <= slv_reg28;
	            slv_reg29 <= slv_reg29;
	            slv_reg30 <= slv_reg30;
	            slv_reg31 <= slv_reg31;
	            slv_reg32 <= slv_reg32;
	            slv_reg33 <= slv_reg33;
	            slv_reg34 <= slv_reg34;
	            slv_reg35 <= slv_reg35;
	            slv_reg36 <= slv_reg36;
	            slv_reg37 <= slv_reg37;
	            slv_reg38 <= slv_reg38;
	            slv_reg39 <= slv_reg39;
	            slv_reg40 <= slv_reg40;
	            slv_reg41 <= slv_reg41;
	            slv_reg42 <= slv_reg42;
	            slv_reg43 <= slv_reg43;
	            slv_reg44 <= slv_reg44;
	            slv_reg45 <= slv_reg45;
	            slv_reg46 <= slv_reg46;
	            slv_reg47 <= slv_reg47;
	            slv_reg48 <= slv_reg48;
	            slv_reg49 <= slv_reg49;
	            slv_reg50 <= slv_reg50;
	            slv_reg51 <= slv_reg51;
	            slv_reg52 <= slv_reg52;
	            slv_reg53 <= slv_reg53;
	            slv_reg54 <= slv_reg54;
	        end case;
	      end if;
	    end if;
	  end if;                   
	end process; 

	-- Implement write response logic generation
	-- The write response and response valid signals are asserted by the slave 
	-- when axi_wready, S_AXI_WVALID, axi_wready and S_AXI_WVALID are asserted.  
	-- This marks the acceptance of address and indicates the status of 
	-- write transaction.

	process (S_AXI_ACLK)
	begin
	  if rising_edge(S_AXI_ACLK) then 
	    if S_AXI_ARESETN = '0' then
	      axi_bvalid  <= '0';
	      axi_bresp   <= "00"; --need to work more on the responses
	    else
	      if (axi_awready = '1' and S_AXI_AWVALID = '1' and axi_wready = '1' and S_AXI_WVALID = '1' and axi_bvalid = '0'  ) then
	        axi_bvalid <= '1';
	        axi_bresp  <= "00"; 
	      elsif (S_AXI_BREADY = '1' and axi_bvalid = '1') then   --check if bready is asserted while bvalid is high)
	        axi_bvalid <= '0';                                 -- (there is a possibility that bready is always asserted high)
	      end if;
	    end if;
	  end if;                   
	end process; 

	-- Implement axi_arready generation
	-- axi_arready is asserted for one S_AXI_ACLK clock cycle when
	-- S_AXI_ARVALID is asserted. axi_awready is 
	-- de-asserted when reset (active low) is asserted. 
	-- The read address is also latched when S_AXI_ARVALID is 
	-- asserted. axi_araddr is reset to zero on reset assertion.

	process (S_AXI_ACLK)
	begin
	  if rising_edge(S_AXI_ACLK) then 
	    if S_AXI_ARESETN = '0' then
	      axi_arready <= '0';
	      axi_araddr  <= (others => '1');
	    else
	      if (axi_arready = '0' and S_AXI_ARVALID = '1') then
	        -- indicates that the slave has acceped the valid read address
	        axi_arready <= '1';
	        -- Read Address latching 
	        axi_araddr  <= S_AXI_ARADDR;           
	      else
	        axi_arready <= '0';
	      end if;
	    end if;
	  end if;                   
	end process; 

	-- Implement axi_arvalid generation
	-- axi_rvalid is asserted for one S_AXI_ACLK clock cycle when both 
	-- S_AXI_ARVALID and axi_arready are asserted. The slave registers 
	-- data are available on the axi_rdata bus at this instance. The 
	-- assertion of axi_rvalid marks the validity of read data on the 
	-- bus and axi_rresp indicates the status of read transaction.axi_rvalid 
	-- is deasserted on reset (active low). axi_rresp and axi_rdata are 
	-- cleared to zero on reset (active low).  
	process (S_AXI_ACLK)
	begin
	  if rising_edge(S_AXI_ACLK) then
	    if S_AXI_ARESETN = '0' then
	      axi_rvalid <= '0';
	      axi_rresp  <= "00";
	    else
	      if (axi_arready = '1' and S_AXI_ARVALID = '1' and axi_rvalid = '0') then
	        -- Valid read data is available at the read data bus
	        axi_rvalid <= '1';
	        axi_rresp  <= "00"; -- 'OKAY' response
	      elsif (axi_rvalid = '1' and S_AXI_RREADY = '1') then
	        -- Read data is accepted by the master
	        axi_rvalid <= '0';
	      end if;            
	    end if;
	  end if;
	end process;

	-- Implement memory mapped register select and read logic generation
	-- Slave register read enable is asserted when valid address is available
	-- and the slave is ready to accept the read address.
	slv_reg_rden <= axi_arready and S_AXI_ARVALID and (not axi_rvalid) ;

	process (axi_araddr,slv_reg_rden)
	variable loc_addr :std_logic_vector(OPT_MEM_ADDR_BITS downto 0);
	begin
	    -- Address decoding for reading registers
	    loc_addr := axi_araddr(ADDR_LSB + OPT_MEM_ADDR_BITS downto ADDR_LSB);
	    case loc_addr is
	      when b"000000" =>
	        reg_data_out <= slv_reg0;
	      when b"000001" =>
	        reg_data_out <= slv_reg1;
	      when b"000010" =>
	        reg_data_out <= slv_reg2;
	      when b"000011" =>
	        reg_data_out <= slv_reg3;
	      when b"000100" =>
	        reg_data_out <= slv_reg4;
	      when b"000101" =>
	        reg_data_out <= slv_reg5;
	      when b"000110" =>
	        reg_data_out <= slv_reg6;
	      when b"000111" =>
	        reg_data_out <= slv_reg7;
	      when b"001000" =>
	        reg_data_out <= slv_reg8;
	      when b"001001" =>
	        reg_data_out <= slv_reg9;
	      when b"001010" =>
	        reg_data_out <= slv_reg10;
	      when b"001011" =>
	        reg_data_out <= slv_reg11;
	      when b"001100" =>
	        reg_data_out <= slv_reg12;
	      when b"001101" =>
	        reg_data_out <= slv_reg13;
	      when b"001110" =>
	        reg_data_out <= slv_reg14;
	      when b"001111" =>
	        reg_data_out <= slv_reg15;
	      when b"010000" =>
	        reg_data_out <= slv_reg16;
	      when b"010001" =>
	        reg_data_out <= slv_reg17;
	      when b"010010" =>
	        reg_data_out <= slv_reg18;
	      when b"010011" =>
	        reg_data_out <= slv_reg19;
	      when b"010100" =>
	        reg_data_out <= slv_reg20;
	      when b"010101" =>
	        reg_data_out <= slv_reg21;
	      when b"010110" =>
	        reg_data_out <= slv_reg22;
	      when b"010111" =>
	        reg_data_out <= slv_reg23;
	      when b"011000" =>
	        reg_data_out <= slv_reg24;
	      when b"011001" =>
	        reg_data_out <= slv_reg25;
	      when b"011010" =>
	        reg_data_out <= slv_reg26;
	      when b"011011" =>
	        reg_data_out <= slv_reg27;
	      when b"011100" =>
	        reg_data_out <= slv_reg28;
	      when b"011101" =>
	        reg_data_out <= slv_reg29;
	      when b"011110" =>
	        reg_data_out <= slv_reg30;
	      when b"011111" =>
	        reg_data_out <= slv_reg31;
	      when b"100000" =>
	        reg_data_out <= slv_reg32;
	      when b"100001" =>
	        reg_data_out <= slv_reg33;
	      when b"100010" =>
	        reg_data_out <= slv_reg34;
	      when b"100011" =>
	        reg_data_out <= slv_reg35;
	      when b"100100" =>
	        reg_data_out <= slv_reg36;
	      when b"100101" =>
	        reg_data_out <= slv_reg37;
	      when b"100110" =>
	        reg_data_out <= slv_reg38;
	      when b"100111" =>
	        reg_data_out <= slv_reg39;
	      when b"101000" =>
	        reg_data_out <= slv_reg40;
	      when b"101001" =>
	        reg_data_out <= slv_reg41;
	      when b"101010" =>
	        reg_data_out <= slv_reg42;
	      when b"101011" =>
	        reg_data_out <= slv_reg43;
	      when b"101100" =>
	        reg_data_out <= slv_reg44;
	      when b"101101" =>
	        reg_data_out <= slv_reg45;
	      when b"101110" =>
	        reg_data_out <= slv_reg46;
	      when b"101111" =>
	        reg_data_out <= slv_reg47;
	      when b"110000" =>
	        reg_data_out <= slv_reg48;
	      when b"110001" =>
	        reg_data_out <= slv_reg49;
	      when b"110010" =>
	        reg_data_out <= slv_reg50;
	      when b"110011" =>
	        reg_data_out <= slv_reg51;
	      when b"110100" =>
	        reg_data_out <= slv_reg52;
	      when b"110101" =>
	        reg_data_out <= slv_reg53;
	      when b"110110" =>
	        reg_data_out <= slv_reg54;
	      when others =>
	        reg_data_out  <= (others => '0');
	    end case;
	end process; 

--	 Output register or memory read data
	process( S_AXI_ACLK ) is
	begin
	  if (rising_edge (S_AXI_ACLK)) then
	    if ( S_AXI_ARESETN = '0' ) then
	      axi_rdata  <= (others => '0');
	    else
	      if (slv_reg_rden = '1') then
	        -- When there is a valid read address (S_AXI_ARVALID) with 
	        -- acceptance of read address by the slave (axi_arready), 
	        -- output the read dada 
	        -- Read address mux
	          axi_rdata <= reg_data_out;     -- register read data
	      end if;   
	    end if;
	  end if;
	end process;



process (S_AXI_ACLK) begin--S_AXI_ACLK=250MHz,CLOK=125MHz
        if (S_AXI_ARESETN='0') then
            CLOK<='1';
            clk_count<=0;
        elsif S_AXI_ACLK'event and S_AXI_ACLK='1' then 
            clk_count<=clk_count+1;
            if clk_count=clk_expand then
                CLOK<=not CLOK;
                clk_count<=0;
            end if;
        end if;
    end process;
main_sb: main_sub
    port map(
    --input--
        clk =>CLOK,
        reset=>'0',
        sig_data_in=>sig_data_in,
        cube_data =>cube_data_temp,
    --output--
        sig_solve_end =>sig_solve_end,--1FvZI
        sig_solve_step=>sig_solve_step);


end arch_imp;
