----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date: 2021/12/24 16:42:20
-- Design Name: 
-- Module Name: calc_cnn - Behavioral
-- Project Name: 
-- Target Devices: 
-- Tool Versions: 
-- Description: 
-- 
-- Dependencies: 
-- 
-- Revision:
-- Revision 0.01 - File Created
-- Additional Comments: calc cnn 4 clock
-- 
----------------------------------------------------------------------------------


library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use ieee.numeric_std.all;

-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
--use IEEE.NUMERIC_STD.ALL;

-- Uncomment the following library declaration if instantiating
-- any Xilinx leaf cells in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity calc_cnn is
    generic (
        in_int : integer;
        in_dec : integer;
        w_int : integer;
        w_dec : integer
    );
    Port (
    --input--
    clk : in STD_LOGIC;
    ch_data : in signed((in_int+in_dec)*9*4 -1 downto 0);--24*36=864bit--
    ch_w : in signed((w_int+w_dec)*9*4 -1 downto 0);
    ch_bias : in signed(w_int+w_dec -1 downto 0);
    --output--
    ch_out : out signed((in_int+in_dec) -1 downto 0) --(1,7,16):o24bit
    );
end calc_cnn;

architecture Behavioral of calc_cnn is

    --1wo--
    --3clock--
    component calc_layer
        generic (
            in_int : integer;
            in_dec : integer;
            w_int : integer;
            w_dec : integer
        );
        Port (
        --input--
            clk : in STD_LOGIC;
            layer_data : in signed((in_int+in_dec)*9 -1 downto 0); --(1,6,10):17bit--
            layer_w : in signed((w_int+w_dec)*9 -1 downto 0); --(1,6,10)--
        --output--
            layer_out : out signed((in_int+in_dec) -1 downto 0) --(1,6,10):o17bit--
        );
    end component;
    
    --0~3w+biasZ--
    --1clock--
    component add_5_data
        generic (
            data_len : integer;
            w_len : integer
        );
        Port (
        --input--
            clk : in STD_LOGIC;
            data_0 : in signed(data_len -1 downto 0) := (others => '0');
            data_1 : in signed(data_len -1 downto 0) := (others => '0');
            data_2 : in signed(data_len -1 downto 0) := (others => '0');
            data_3 : in signed(data_len -1 downto 0) := (others => '0');
            bias : in signed(w_len -1 downto 0) := (others => '0');
        --output--
            add_out : out signed(data_len -1 downto 0)
        );
    end component;
        
    constant data_len : integer := in_int+in_dec;
    constant w_len : integer := w_int+w_dec;
     
    --M--
    --f[^--
    signal layer_0_data : signed(data_len*9 -1 downto 0);
    signal layer_1_data : signed(data_len*9 -1 downto 0);
    signal layer_2_data : signed(data_len*9 -1 downto 0);
    signal layer_3_data : signed(data_len*9 -1 downto 0);
    --d--
    signal layer_0_w : signed(w_len*9 -1 downto 0);
    signal layer_1_w : signed(w_len*9 -1 downto 0);
    signal layer_2_w : signed(w_len*9 -1 downto 0);
    signal layer_3_w : signed(w_len*9 -1 downto 0);
    --layero--
    signal layer_0_out : signed(data_len -1 downto 0);--Z̏_Zp
    signal layer_1_out : signed(data_len -1 downto 0);
    signal layer_2_out : signed(data_len -1 downto 0);
    signal layer_3_out : signed(data_len -1 downto 0);
    --bias--
    type bias_type is array(0 to 2) of signed(w_len -1 downto 0);
    signal bias_temp : bias_type := (others => (others => '0'));
    --ZM--
--    signal add_out : signed(data_len+w_int+6 -1 downto 0) := (others => '0');
    signal add_out : signed(data_len -1 downto 0) := (others => '0');--Z̏_Zp
begin
    --M--
    layer_0_data <= ch_data(ch_data'length -1 downto ch_data'length-(data_len)*9*1);
    layer_1_data <= ch_data(ch_data'length-(data_len)*9*1 -1 downto ch_data'length-(data_len)*9*2);
    layer_2_data <= ch_data(ch_data'length-(data_len)*9*2 -1 downto ch_data'length-(data_len)*9*3);
    layer_3_data <= ch_data(ch_data'length-(data_len)*9*3 -1 downto ch_data'length-(data_len)*9*4);
    layer_0_w <= ch_w(ch_w'length -1 downto ch_w'length-(w_len)*9*1);
    layer_1_w <= ch_w(ch_w'length-(w_len)*9*1 -1 downto ch_w'length-(w_len)*9*2);
    layer_2_w <= ch_w(ch_w'length-(w_len)*9*2 -1 downto ch_w'length-(w_len)*9*3);
    layer_3_w <= ch_w(ch_w'length-(w_len)*9*3 -1 downto ch_w'length-(w_len)*9*4);
    
    --biasx--
    --3NbN--
    process(clk)
    begin
        if (clk'event and clk='1') then
            bias_temp(0) <= ch_bias;
            bias_temp(1) <= bias_temp(0);
            bias_temp(2) <= bias_temp(1);
        end if;
    end process;
    
    --layer_0--
    layer_0 : calc_layer
        generic map (
            in_int => in_int,
            in_dec => in_dec,
            w_int => w_int,
            w_dec => w_dec
        )
        port map (
            clk => clk,
            layer_data => layer_0_data,
            layer_w => layer_0_w,
            layer_out => layer_0_out
        );
    
    --layer_1--
    layer_1 : calc_layer
        generic map (
            in_int => in_int,
            in_dec => in_dec,
            w_int => w_int,
            w_dec => w_dec
        )
        port map (
            clk => clk,
            layer_data => layer_1_data,
            layer_w => layer_1_w,
            layer_out => layer_1_out
        );
    
    --layer_2--
    layer_2 : calc_layer
        generic map (
            in_int => in_int,
            in_dec => in_dec,
            w_int => w_int,
            w_dec => w_dec
        )
        port map (
            clk => clk,
            layer_data => layer_2_data,
            layer_w => layer_2_w,
            layer_out => layer_2_out
        );
    
    --layer_3--
    layer_3 : calc_layer
        generic map (
            in_int => in_int,
            in_dec => in_dec,
            w_int => w_int,
            w_dec => w_dec
        )
        port map (
            clk => clk,
            layer_data => layer_3_data,
            layer_w => layer_3_w,
            layer_out => layer_3_out
        );
    
    --layer out--
    layer_out : add_5_data
        generic map (
            data_len => data_len,
            w_len => w_len
        )
        port map (
            clk => clk,
            data_0 => layer_0_out, --(1,6,10)--
            data_1 => layer_1_out,
            data_2 => layer_2_out,
            data_3 => layer_3_out,
            bias => bias_temp(2),
            add_out => ch_out --(1,6,10)--
        );    

    
end Behavioral;