----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date: 2022/01/10 23:02:32
-- Design Name: 
-- Module Name: cube_change - Behavioral
-- Project Name: 
-- Target Devices: 
-- Tool Versions: 
-- Description: 
-- 
-- Dependencies: 
-- 
-- Revision:
-- Revision 0.01 - File Created
-- Additional Comments:
-- 
----------------------------------------------------------------------------------


library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use ieee.numeric_std.all;

-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
--use IEEE.NUMERIC_STD.ALL;

-- Uncomment the following library declaration if instantiating
-- any Xilinx leaf cells in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity cube_change is
    Port (
    --input--
        clk : in STD_LOGIC;
        reset : in std_logic;
        sig_change : in std_logic;
        move_step : in unsigned(3 downto 0);
        current_cube : in unsigned(99 downto 0);
    --output--
        sig_update_cube : out std_logic;
        update_cube : out unsigned(99 downto 0)
    );
end cube_change;

architecture Behavioral of cube_change is

    --internal signal--
    type corner is array(0 to 7) of unsigned(4 downto 0);
    type edge is array(0 to 11) of unsigned(4 downto 0);
    signal corner_i : corner := (others => (others => '0'));
    signal corner_work : corner := (others => (others => '0'));
    signal edge_i : edge := (others => (others => '0'));
    signal edge_work : edge := (others => (others => '0'));
    
    --store corner data--
    function conv_corner_data (in_data : unsigned(99 downto 0) := (others => '0')) return corner is
        variable corner_func : corner := (others => (others => '0'));
    begin
        for corner_num in 0 to 7 loop
            corner_func(corner_num) := in_data(in_data'length-5*corner_num -1 downto in_data'length-5*(corner_num+1));
        end loop;
        
        return corner_func;
    end function;
    
    --store edge data--
    function conv_edge_data (in_data : unsigned(99 downto 0) := (others => '0')) return edge is
        variable edge_func : edge := (others => (others => '0'));
    begin
        for edge_num in 0 to 11 loop
            edge_func(edge_num) := in_data(in_data'length-5*(8+edge_num) -1 downto in_data'length-5*(8+edge_num+1));
        end loop;
        
        return edge_func;
    end function;
    
    --correct corner dir/change data_type--
    function store_data (in_corner : corner := (others => (others => '0'));
                         in_edge : edge := (others => (others => '0'))) return unsigned is
        variable cube_func : unsigned(99 downto 0) := (others => '0');
    begin
        --corr corner_dir/store corner data--
        for corner_num in 0 to 7 loop
            if (in_corner(corner_num)(1 downto 0) = "11") then
                cube_func(cube_func'length-5*corner_num -1 downto cube_func'length-5*(corner_num+1)) := in_corner(corner_num)(4 downto 2) & "00";
            else
                cube_func(cube_func'length-5*corner_num -1 downto cube_func'length-5*(corner_num+1)) := in_corner(corner_num);
            end if;
        end loop;
        
        --store edge data--
        for edge_num in 0 to 11 loop
            cube_func(cube_func'length-5*(8+edge_num) -1 downto cube_func'length-5*(8+edge_num+1)) := in_edge(edge_num);
        end loop;
        
        return cube_func;
    end function;
    
begin
    --store corner/edge data--
    corner_i <= conv_corner_data(current_cube);
    edge_i <= conv_edge_data(current_cube);

    --output update data--
    update_cube <= store_data(corner_work, edge_work);
    
    process(clk) begin
        if clk'event and clk='1' then
            if reset ='1' then
                corner_work <= (others => (others => '0'));
                edge_work <= (others => (others => '0'));
--                for corner_num in 0 to 7 loop
--                    corner_work(corner_num) <= to_unsigned(corner_num, 3) & "00";
--                end loop;
--                for edge_num in 0 to 11 loop
--                    edge_work(edge_num) <= to_unsigned(edge_num, 4) & '0';
--                end loop;
            elsif sig_change = '1' then
                case move_step is
                    when "0001" =>--move=0001:U
                        corner_work(0) <= corner_i(3); corner_work(1) <= corner_i(0); corner_work(2) <= corner_i(1); corner_work(3) <= corner_i(2);--corner:0-3
                        
                        edge_work(0) <= edge_i(3); edge_work(1) <= edge_i(0); edge_work(2) <= edge_i(1); edge_work(3) <= edge_i(2);--edge:0-3
                        
                        for num in 4 to 7 loop
                            corner_work(num) <= corner_i(num);
                            edge_work(num) <= edge_i(num);
                            edge_work(num+4) <= edge_i(num+4);
                        end loop;
                    when "0010" =>--move=0010:UP
                        corner_work(0) <= corner_i(1); corner_work(1) <= corner_i(2); corner_work(2) <= corner_i(3); corner_work(3) <= corner_i(0);--corner:0-3
                        
                        edge_work(0) <= edge_i(1); edge_work(1) <= edge_i(2); edge_work(2) <= edge_i(3); edge_work(3) <= edge_i(0);--edge:0-3
                        
                        for num in 4 to 7 loop
                            corner_work(num) <= corner_i(num);
                            edge_work(num) <= edge_i(num);
                            edge_work(num+4) <= edge_i(num+4);
                        end loop;
                    when "0011" =>--move=0011:D
                        corner_work(4) <= corner_i(5); corner_work(5) <= corner_i(6); corner_work(6) <= corner_i(7); corner_work(7) <= corner_i(4);--corner:0-3
                        
                        edge_work(8) <= edge_i(9); edge_work(9) <= edge_i(10); edge_work(10) <= edge_i(11); edge_work(11) <= edge_i(8);--edge:8-11
                        
                        for num in 0 to 3 loop
                            corner_work(num) <= corner_i(num);
                            edge_work(num) <= edge_i(num);
                            edge_work(num+4) <= edge_i(num+4);
                        end loop;
                    when "0100" =>--move=0100:DP
                        corner_work(4) <= corner_i(7); corner_work(5) <= corner_i(4); corner_work(6) <= corner_i(5); corner_work(7) <= corner_i(6);--corner:0-3
                        
                        edge_work(8) <= edge_i(11); edge_work(9) <= edge_i(8); edge_work(10) <= edge_i(9); edge_work(11) <= edge_i(10);--edge:0-3
                        
                        for num in 0 to 3 loop
                            corner_work(num) <= corner_i(num);
                            edge_work(num) <= edge_i(num);
                            edge_work(num+4) <= edge_i(num+4);
                        end loop;
                    when "0101" =>--move=0101:F
                        corner_work(2) <= corner_i(3)(4 downto 2) & (corner_i(3)(1 downto 0) + to_unsigned(1,2));--corner:2
                        corner_work(3) <= corner_i(7)(4 downto 2) & (corner_i(7)(1 downto 0) + to_unsigned(2,2));--corner:3
                        corner_work(6) <= corner_i(2)(4 downto 2) & (corner_i(2)(1 downto 0) + to_unsigned(2,2));--corner:6
                        corner_work(7) <= corner_i(6)(4 downto 2) & (corner_i(6)(1 downto 0) + to_unsigned(1,2));--corner:7
                        
                        edge_work(2) <= edge_i(7)(4 downto 1) & (edge_i(7)(0 downto 0) + to_unsigned(1,1));--edge:2
                        edge_work(6) <= edge_i(2)(4 downto 1) & (edge_i(2)(0 downto 0) + to_unsigned(1,1));--edge:6
                        edge_work(7) <= edge_i(10)(4 downto 1) & (edge_i(10)(0 downto 0) + to_unsigned(1,1));--edge:7
                        edge_work(10) <= edge_i(6)(4 downto 1) & (edge_i(6)(0 downto 0) + to_unsigned(1,1));--edge:10
                        
                        for num in 0 to 1 loop
                            corner_work(num) <= corner_i(num);
                            corner_work(num+4) <= corner_i(num+4);
                            edge_work(num) <= edge_i(num);
                            edge_work(num+4) <= edge_i(num+4);
                            edge_work(num+8) <= edge_i(num+8);
                        end loop;
                        edge_work(3) <= edge_i(3);
                        edge_work(11) <= edge_i(11);
                    when "0110" =>--move=0110:FP
                        corner_work(2) <= corner_i(6)(4 downto 2) & (corner_i(6)(1 downto 0) + to_unsigned(1,2));--corner:2
                        corner_work(3) <= corner_i(2)(4 downto 2) & (corner_i(2)(1 downto 0) + to_unsigned(2,2));--corner:3
                        corner_work(6) <= corner_i(7)(4 downto 2) & (corner_i(7)(1 downto 0) + to_unsigned(2,2));--corner:6
                        corner_work(7) <= corner_i(3)(4 downto 2) & (corner_i(3)(1 downto 0) + to_unsigned(1,2));--corner:7
                        
                        edge_work(2) <= edge_i(6)(4 downto 1) & (edge_i(6)(0 downto 0) + to_unsigned(1,1));--edge:2
                        edge_work(6) <= edge_i(10)(4 downto 1) & (edge_i(10)(0 downto 0) + to_unsigned(1,1));--edge:6
                        edge_work(7) <= edge_i(2)(4 downto 1) & (edge_i(2)(0 downto 0) + to_unsigned(1,1));--edge:7
                        edge_work(10) <= edge_i(7)(4 downto 1) & (edge_i(7)(0 downto 0) + to_unsigned(1,1));--edge:10
                        
                        for num in 0 to 1 loop
                            corner_work(num) <= corner_i(num);
                            corner_work(num+4) <= corner_i(num+4);
                            edge_work(num) <= edge_i(num);
                            edge_work(num+4) <= edge_i(num+4);
                            edge_work(num+8) <= edge_i(num+8);
                        end loop;
                        edge_work(3) <= edge_i(3);
                        edge_work(11) <= edge_i(11);
                    when "0111" =>--move=0111:B
                        corner_work(0) <= corner_i(1)(4 downto 2) & (corner_i(1)(1 downto 0) + to_unsigned(1,2));--corner:0
                        corner_work(1) <= corner_i(5)(4 downto 2) & (corner_i(5)(1 downto 0) + to_unsigned(2,2));--corner:1
                        corner_work(4) <= corner_i(0)(4 downto 2) & (corner_i(0)(1 downto 0) + to_unsigned(2,2));--corner:4
                        corner_work(5) <= corner_i(4)(4 downto 2) & (corner_i(4)(1 downto 0) + to_unsigned(1,2));--corner:5
                        
                        edge_work(0) <= edge_i(5)(4 downto 1) & (edge_i(5)(0 downto 0) + to_unsigned(1,1));--edge:0
                        edge_work(4) <= edge_i(0)(4 downto 1) & (edge_i(0)(0 downto 0) + to_unsigned(1,1));--edge:4
                        edge_work(5) <= edge_i(8)(4 downto 1) & (edge_i(8)(0 downto 0) + to_unsigned(1,1));--edge:5
                        edge_work(8) <= edge_i(4)(4 downto 1) & (edge_i(4)(0 downto 0) + to_unsigned(1,1));--edge:8
                        
                        for num in 0 to 1 loop
                            corner_work(num+2) <= corner_i(num+2);
                            corner_work(num+6) <= corner_i(num+6);
                            edge_work(num+1) <= edge_i(num+1);
                            edge_work(num+6) <= edge_i(num+6);
                            edge_work(num+9) <= edge_i(num+9);
                        end loop;
                        edge_work(3) <= edge_i(3);
                        edge_work(11) <= edge_i(11);
                    when "1000" =>--move=1000:BP
                        corner_work(0) <= corner_i(4)(4 downto 2) & (corner_i(4)(1 downto 0) + to_unsigned(1,2));--corner:0
                        corner_work(1) <= corner_i(0)(4 downto 2) & (corner_i(0)(1 downto 0) + to_unsigned(2,2));--corner:1
                        corner_work(4) <= corner_i(5)(4 downto 2) & (corner_i(5)(1 downto 0) + to_unsigned(2,2));--corner:4
                        corner_work(5) <= corner_i(1)(4 downto 2) & (corner_i(1)(1 downto 0) + to_unsigned(1,2));--corner:5
                        
                        edge_work(0) <= edge_i(4)(4 downto 1) & (edge_i(4)(0 downto 0) + to_unsigned(1,1));--edge:0
                        edge_work(4) <= edge_i(8)(4 downto 1) & (edge_i(8)(0 downto 0) + to_unsigned(1,1));--edge:4
                        edge_work(5) <= edge_i(0)(4 downto 1) & (edge_i(0)(0 downto 0) + to_unsigned(1,1));--edge:5
                        edge_work(8) <= edge_i(5)(4 downto 1) & (edge_i(5)(0 downto 0) + to_unsigned(1,1));--edge:8
                        
                        for num in 0 to 1 loop
                            corner_work(num+2) <= corner_i(num+2);
                            corner_work(num+6) <= corner_i(num+6);
                            edge_work(num+1) <= edge_i(num+1);
                            edge_work(num+6) <= edge_i(num+6);
                            edge_work(num+9) <= edge_i(num+9);
                        end loop;
                        edge_work(3) <= edge_i(3);
                        edge_work(11) <= edge_i(11);
                    when "1001" =>--move=1001:R
                        corner_work(1) <= corner_i(2)(4 downto 2) & (corner_i(2)(1 downto 0) + to_unsigned(1,2));--corner:1
                        corner_work(2) <= corner_i(6)(4 downto 2) & (corner_i(6)(1 downto 0) + to_unsigned(2,2));--corner:2
                        corner_work(5) <= corner_i(1)(4 downto 2) & (corner_i(1)(1 downto 0) + to_unsigned(2,2));--corner:5
                        corner_work(6) <= corner_i(5)(4 downto 2) & (corner_i(5)(1 downto 0) + to_unsigned(1,2));--corner:6
                        
                        edge_work(1) <= edge_i(6); edge_work(5) <= edge_i(1); edge_work(6) <= edge_i(9); edge_work(9) <= edge_i(5);--edge:1,5,6,9
                        
                        for num in 0 to 1 loop
                            corner_work(num+3) <= corner_i(num+3);
                            edge_work(num+2) <= edge_i(num+2);
                            edge_work(num+7) <= edge_i(num+7);
                            edge_work(num+10) <= edge_i(num+10);
                        end loop;
                        corner_work(0) <= corner_i(0);
                        corner_work(7) <= corner_i(7);
                        edge_work(0) <= edge_i(0);
                        edge_work(4) <= edge_i(4);
                    when "1010" =>--move=1010:RP
                        corner_work(1) <= corner_i(5)(4 downto 2) & (corner_i(5)(1 downto 0) + to_unsigned(1,2));--corner:1
                        corner_work(2) <= corner_i(1)(4 downto 2) & (corner_i(1)(1 downto 0) + to_unsigned(2,2));--corner:2
                        corner_work(5) <= corner_i(6)(4 downto 2) & (corner_i(6)(1 downto 0) + to_unsigned(2,2));--corner:5
                        corner_work(6) <= corner_i(2)(4 downto 2) & (corner_i(2)(1 downto 0) + to_unsigned(1,2));--corner:6
                        
                        edge_work(1) <= edge_i(5); edge_work(5) <= edge_i(9); edge_work(6) <= edge_i(1); edge_work(9) <= edge_i(6);--edge:1,5,6,9
                        
                        for num in 0 to 1 loop
                            corner_work(num+3) <= corner_i(num+3);
                            edge_work(num+2) <= edge_i(num+2);
                            edge_work(num+7) <= edge_i(num+7);
                            edge_work(num+10) <= edge_i(num+10);
                        end loop;
                        corner_work(0) <= corner_i(0);
                        corner_work(7) <= corner_i(7);
                        edge_work(0) <= edge_i(0);
                        edge_work(4) <= edge_i(4);
                    when "1011" =>--move=1011:L
                        corner_work(0) <= corner_i(4)(4 downto 2) & (corner_i(4)(1 downto 0) + to_unsigned(1,2));--corner:0
                        corner_work(3) <= corner_i(0)(4 downto 2) & (corner_i(0)(1 downto 0) + to_unsigned(2,2));--corner:3
                        corner_work(4) <= corner_i(7)(4 downto 2) & (corner_i(7)(1 downto 0) + to_unsigned(2,2));--corner:4
                        corner_work(7) <= corner_i(3)(4 downto 2) & (corner_i(3)(1 downto 0) + to_unsigned(1,2));--corner:7
                        
                        edge_work(3) <= edge_i(4); edge_work(4) <= edge_i(11); edge_work(7) <= edge_i(3); edge_work(11) <= edge_i(7);--edge:3,4,7,11
                        
                        for num in 0 to 1 loop
                            corner_work(num+1) <= corner_i(num+1);
                            corner_work(num+5) <= corner_i(num+5);
                            edge_work(num+1) <= edge_i(num+1);
                            edge_work(num+5) <= edge_i(num+5);
                            edge_work(num+8) <= edge_i(num+8);
                        end loop;
                        edge_work(0) <= edge_i(0);
                        edge_work(10) <= edge_i(10);
                    when "1100" =>--move=1100:LP
                        corner_work(0) <= corner_i(3)(4 downto 2) & (corner_i(3)(1 downto 0) + to_unsigned(1,2));--corner:0
                        corner_work(3) <= corner_i(7)(4 downto 2) & (corner_i(7)(1 downto 0) + to_unsigned(2,2));--corner:3
                        corner_work(4) <= corner_i(0)(4 downto 2) & (corner_i(0)(1 downto 0) + to_unsigned(2,2));--corner:4
                        corner_work(7) <= corner_i(4)(4 downto 2) & (corner_i(4)(1 downto 0) + to_unsigned(1,2));--corner:7
                        
                        edge_work(3) <= edge_i(7); edge_work(4) <= edge_i(3); edge_work(7) <= edge_i(11); edge_work(11) <= edge_i(4);--edge:3,4,7,11
                        
                        for num in 0 to 1 loop
                            corner_work(num+1) <= corner_i(num+1);
                            corner_work(num+5) <= corner_i(num+5);
                            edge_work(num+1) <= edge_i(num+1);
                            edge_work(num+5) <= edge_i(num+5);
                            edge_work(num+8) <= edge_i(num+8);
                        end loop;
                        edge_work(0) <= edge_i(0);
                        edge_work(10) <= edge_i(10);
                    when others =>
                        corner_work <= corner_i;
                        edge_work <= edge_i;
                end case;
            end if;
        end if;
    end process;
    
    --DFF--
    process(clk)
    begin
        if (clk'event and clk='1') then
            sig_update_cube <= sig_change;
        end if;
    end process;

end Behavioral;
