----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date: 2021/12/31 13:37:32
-- Design Name: 
-- Module Name: eLU_function - Behavioral
-- Project Name: 
-- Target Devices: 
-- Tool Versions: 
-- Description: 
-- 
-- Dependencies: 
-- 
-- Revision:
-- Revision 0.01 - File Created
-- Additional Comments:
-- 
----------------------------------------------------------------------------------


library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use ieee.numeric_std.all;

-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
--use IEEE.NUMERIC_STD.ALL;

-- Uncomment the following library declaration if instantiating
-- any Xilinx leaf cells in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity eLU_function is
    generic (
        data_int : integer;
        data_dec : integer
    );
    Port (
    --input--
        clk : in STD_LOGIC;
        eLU_in : in signed(data_int+data_dec -1 downto 0);
    --output--
        eLU_out : out signed(data_int+data_dec -1 downto 0)
    );
end eLU_function;

architecture Behavioral of eLU_function is

    component eLU_table
        Port (
        --input--
            clk : in STD_LOGIC;
            ena : in std_logic;
            addr : in signed(10 downto 0); --1597 < 2048=2^12
        --output--
            eLU_out : out signed(7 downto 0) --o͂͏8bit
        );
    end component;

    constant data_len : integer := data_int+data_dec;
    
    --internal signal--
    signal temp_0, temp_1 : signed(data_len -1 downto 0) := (others => '0');
    signal temp_msb : std_logic := '0';
    signal ena : std_logic := '0';
    signal temp_addr : signed(11 downto 0) := (others => '0');
    signal table_in : signed(10 downto 0) := (others => '0');
    signal table_out : signed(7 downto 0) := (others => '0');
    signal round_data : signed(14 downto 0) := (others => '0');

    function round (msb : std_logic := '0';
             eLU_temp : signed(data_len -1 downto 0)) return signed is
        variable round_temp : signed(14 downto 0);
    begin
        if (msb = '1' and eLU_temp(1) = '1') then --Jオ肠
            round_temp := eLU_temp(eLU_temp'length -1 downto 2) + to_signed(1, round_temp'length);
        elsif (msb = '1' and eLU_temp(1) = '0') then
            round_temp := eLU_temp(eLU_temp'length -1 downto 2);
        else
            round_temp := (others => '0');
        end if;
        return round_temp;
    end round;
begin

    eLU_data : eLU_table
        port map (
        --input--
            clk => clk,
            ena => ena,
            addr => table_in,
        --output--
            eLU_out => table_out
        );
    
    --function--
    temp_msb <= eLU_in(eLU_in'length -1);--ŏʃrbgo
    
    round_data <= round(temp_msb, eLU_in);
    
    --conv_eLU--
    process(clk)
    begin
        if (clk'event and clk='1') then
            if (temp_msb = '0') then
                temp_0 <= eLU_in;
                ena <= '0';
            else
                if (to_integer(round_data) < -1597) then --:-1598(8bitZ-6.2421875,11111001.11000010)ȉ-1
                    temp_0 <= to_signed(130048, data_len); --(1,6,10)-1
                    ena <= '0';
                else
                    temp_addr <= resize(round_data, temp_addr'length) + to_signed(1597, temp_addr'length -1); --AhXϊ
                    ena <= '1';
                    temp_0 <= (others => '0');
                end if;
            end if;
        end if;
    end process;
    table_in <= temp_addr(temp_addr'length-1 -1 downto 0);

    --DFF--
    process(clk)
    begin
        if (clk'event and clk='1') then
            temp_1 <= temp_0;
        end if;
    end process;
    
    --output--
    process(clk)
    begin
        if (clk'event and clk='1') then
            if (to_integer(table_out) /= 0) then
                eLU_out <= to_signed(127, 7) & table_out & to_signed(0, 2); --1111111, 00ǉ
            else
                eLU_out <= temp_1;
            end if;
        end if;
    end process;

end Behavioral;
