----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date: 2022/01/14 16:40:36
-- Design Name: 
-- Module Name: forward_re - Behavioral
-- Project Name: 
-- Target Devices: 
-- Tool Versions: 
-- Description: 
-- 
-- Dependencies: 
-- 
-- Revision:
-- Revision 0.01 - File Created
-- Additional Comments:
-- 
----------------------------------------------------------------------------------


library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use ieee.numeric_std.all;

-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
--use IEEE.NUMERIC_STD.ALL;

-- Uncomment the following library declaration if instantiating
-- any Xilinx leaf cells in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity forward_re is
    Port (
    --input--
        clk : in STD_LOGIC;
        reset : in std_logic;
        cube_data : in unsigned(99 downto 0);
    --output--
        sig_move_out : out std_logic;
        move_num : out unsigned(3 downto 0)
    );
end forward_re;

architecture Behavioral of forward_re is

    --counter522:layer1_calc_counter--
    component counter_layer1
        Port (
        --input--
            clk : in STD_LOGIC;
            reset : in std_logic;
        --output--
            count : out unsigned(9 downto 0)
        );
    end component;
    
    --counter3501:layer2_calc_counter--
    component counter_layer2
        Port (
        --input--
            clk : in STD_LOGIC;
            reset : in std_logic;
        --output--
            count : out unsigned(11 downto 0)
        );
    end component;
    
    --counter207:layer_affine_calc_counter--
    component counter_layer_affine
        Port (
        --input--
            clk : in STD_LOGIC;
            reset : in std_logic;
        --output--
            count : out unsigned(7 downto 0)
        );
    end component;
    
    --zero padding data : for calc layer_1--
    component padding_data_re
        generic (
            data_int : integer;
            data_dec : integer
        );
        Port (
        --input--
            clk : in STD_LOGIC;
            reset : in std_logic;
            count : in unsigned(9 downto 0);
            in_data : in signed(data_int+data_dec -1 downto 0);
        --output--
            sig_36_data_out : out std_logic;
            uni_data : out signed((data_int+data_dec)*36 -1 downto 0)
        );    
    end component;
    
    --zero padding data : for calc layer_2--
    component padding_calc_out_1_re
        generic (
            data_int : integer;
            data_dec : integer
        );
        Port (
        --input--
            clk : in STD_LOGIC;
            reset_layer1 : in std_logic;
            sig_calc_end : in std_logic;
            reset_layer2 : in std_logic;
            count_layer2 : in unsigned(11 downto 0);
            in_data : in signed((data_int+data_dec) -1 downto 0);
        --output--
            sig_36_data_out : out std_logic;
            sig_uni_karnel_out : out std_logic;
            uni_data : out signed((data_int+data_dec)*36 -1 downto 0)
        );
    end component;
    
    --zero padding data : for calc layer3--
    component padding_calc_out_2_re
    generic (
        data_int : integer;
        data_dec : integer
    );
    Port (
    --input--
        clk : in STD_LOGIC;
        reset_layer2 : in std_logic;
        sig_calc_out : in std_logic;
        reset_layer3 : in std_logic;
        count_layer3 : in unsigned(11 downto 0);
        in_data : in signed((data_int+data_dec) -1 downto 0);
    --output--
        sig_36_data_out : out std_logic;
        sig_uni_karnel_out : out std_logic;
        uni_data : out signed((data_int+data_dec)*36 -1 downto 0)
    );
    end component;
    
    --store layer4 data--
    component store_layer4_re
        generic (
            data_int : integer;
            data_dec : integer
        );
        Port (
        --input--
            clk : in STD_LOGIC;
            reset_layer4 : in std_logic;
            sig_calc_out : in std_logic;
            reset_layer_affine : in std_logic;
            count_layer_affine : in unsigned(7 downto 0);
            in_data : in signed((data_int+data_dec) -1 downto 0);
        --output--
            sig_36_data_out : out std_logic;
            sig_uni_data_out : out std_logic;
            uni_data_a : out signed((data_int+data_dec)*36 -1 downto 0);
            uni_data_b : out signed((data_int+data_dec)*36 -1 downto 0)
        );
    end component;
    
    --compare q_value--
    component compare_q_value
        generic (
            data_len : integer
        );
        Port (
        --input--
            clk : in STD_LOGIC;
            sig_in : in std_logic;
            in_data : in signed(data_len*12 -1 downto 0);
        --output--
            move_num : out unsigned(3 downto 0)
        );
    end component;
    
    --generate weight addr--    
    component gen_w_addr
        Port (
        --input--
            clk : in STD_LOGIC;
            reset_layer1 : in std_logic;
            reset_layer2 : in std_logic;
            reset_layer3 : in std_logic;
            reset_layer4 : in std_logic;
            reset_layer_affine : in std_logic;
            count_layer1 : in unsigned(9 downto 0);
            count_layer2 : in unsigned(11 downto 0);
            count_layer3 : in unsigned(11 downto 0);
            count_layer4 : in unsigned(11 downto 0);
            count_layer_affine : in unsigned(7 downto 0);
        --output--
            bias_rom_en : out std_logic;
            bias_addr : out unsigned(7 downto 0);
            w_rom_en : out unsigned(31 downto 0); --rom_en, 32ch
            w_addr : out unsigned(10 downto 0) --d݃f[^ő1092<2048=2^11
        );
    end component;

    
    --calulation unit:CNN--
    component calc_cnn
        generic (
            in_int : integer;
            in_dec : integer;
            w_int : integer;
            w_dec : integer
        );
        Port (
        --input--
            clk : in STD_LOGIC;
            ch_data : in signed((in_int+in_dec)*9*4 -1 downto 0);--576bit--
            ch_w : in signed((w_int+w_dec)*9*4 -1 downto 0);
            ch_bias : in signed(w_int+w_dec -1 downto 0);
        --output--
            ch_out : out signed((in_int+in_dec) -1 downto 0) --add_5_data
        );
    end component;
    
    --eLU_function--
    component eLU_function
    generic (
        data_int : integer;
        data_dec : integer
    );
    Port (
    --input--
        clk : in STD_LOGIC;
        eLU_in : in signed(data_int+data_dec -1 downto 0);
    --output--
        eLU_out : out signed(data_int+data_dec -1 downto 0)
    );
    end component;
        
    --w_rom:layer1_weight_data--
    component w_rom
        generic (
            file_name : string;
            data_len : integer;
            addr_bit : integer
        );
        Port (
        --input--
            clk : in STD_LOGIC;
            ena : in std_logic;
            addr : in unsigned(addr_bit -1 downto 0);
        --output--
            dout : out signed(data_len -1 downto 0)
        );
    end component;
    
    --bias_rom:bias_data--
    component bias_rom
        generic (
            file_name : string;
            data_len : integer;
            addr_bit : integer
        );
        Port (
        --input--
            clk : in STD_LOGIC;
            ena : in std_logic;
            addr : in unsigned(addr_bit -1 downto 0);
        --output--
            dout : out signed(data_len -1 downto 0)
        );
    end component;

    
    --internal signal--
    constant data_int : integer := 7;
    constant data_dec : integer := 10;
    constant data_len : integer := data_int+data_dec; --24bit
    constant w_int : integer := 7;
    constant w_dec : integer := 10;
    constant w_len : integer := w_int+w_dec; --24bit
    constant init_rom_addr_bit : integer := 6;
    constant w_rom_addr_bit : integer := 11;
    constant bias_addr_bit : integer := 8;
    
    --for counter--
    --layer_1--
    signal reset_layer1 : std_logic := '0';
    signal count_layer1 : unsigned(9 downto 0) := (others => '0');
    --layer_2--
    signal reset_layer2 : std_logic := '0';
    signal count_layer2 : unsigned(11 downto 0) := (others => '0');
    --layer_3--
    signal reset_layer3 : std_logic := '0';
    signal count_layer3 : unsigned(11 downto 0) := (others => '0');
    --layer_4--
    signal reset_layer4 : std_logic := '0';
    signal count_layer4 : unsigned(11 downto 0) := (others => '0');
    --layer_affine--
    signal reset_layer_affine : std_logic := '0';
    signal count_layer_affine : unsigned(7 downto 0) := (others => '0');
    --for w_rom--
    signal w_rom_en : unsigned(31 downto 0) := (others => '0');
    signal w_rom_addr : unsigned(w_rom_addr_bit -1 downto 0) := (others => '0');
    type w_data_type is array (0 to 31) of signed(w_len -1 downto 0);
    signal w_rom_data : w_data_type := (others => (others => '0')); --[data_len]bit*32ch array
    --for bias_rom--
    signal bias_rom_en : std_logic := '0';
    signal bias_addr : unsigned(bias_addr_bit -1 downto 0) := (others => '0');
    signal bias_addr_delay : unsigned(bias_addr_bit -1 downto 0) := (others => '0');
    signal bias_temp : signed(w_len -1 downto 0) := (others => '0');
    type bias_type is array(0 to 31) of signed(w_len -1 downto 0);
    signal bias_data, bias_data_temp : bias_type := (others => (others => '0'));
    --for batch reading weight data --
    type uni_karnel_w_type is array (0 to 31) of signed(w_len*36 -1 downto 0);
    signal uni_karnel_w : uni_karnel_w_type := (others => (others => '0'));
    signal uni_karnel_w_temp : uni_karnel_w_type := (others => (others => '0'));
    signal w_rom_addr_delay : unsigned(w_rom_addr_bit -1 downto 0) := (others => '0');
    --for cnn--
    type calc_out_type is array(0 to 31) of signed(data_len -1 downto 0); --(1,7,16)
    signal calc_out : calc_out_type := (others => (others => '0'));
    signal calc_out_delay : calc_out_type := (others => (others => '0'));
    signal uni_karnel_data : uni_karnel_w_type := (others => (others => '0'));
    --for eLU_function--
    signal eLU_out : signed(data_len -1 downto 0) := (others => '0');
    --add 8 ch_out data:layer2/3/4--
    signal add_ch_out : calc_out_type := (others => (others => '0'));
    --for zero padding 1st layer calc out--
    signal sig_layer1_karnel_out : std_logic := '0';
    signal sig_delay_layer1_karnel_out : unsigned(9 downto 0) := (others => '0');
    signal uni_karnel_data_layer1 : signed(data_len*36 -1 downto 0) := (others => '0');
    --for zero padding 2nd layer calc out--
    signal uni_karnel_data_layer2 : signed(data_len*36 -1 downto 0) := (others => '0');
    signal sig_layer2_36_data_out : std_logic := '0';
    signal sig_layer2_karnel_out : std_logic := '0';
    signal sig_delay_layer2_karnel_out : unsigned(9 downto 0) := (others => '0');
    --for zero padding 3rd layer--
    signal uni_karnel_data_layer3 : signed(data_len*36 -1 downto 0) := (others => '0');
    signal sig_layer3_36_data_out : std_logic := '0';
    signal sig_layer3_karnel_out : std_logic := '0';
    signal sig_delay_layer3_karnel_out : unsigned(9 downto 0) := (others => '0');
    --for zero padding 4th layer--
    signal uni_karnel_data_layer4 : signed(data_len*36 -1 downto 0) := (others => '0');
    signal sig_layer4_36_data_out : std_logic := '0';
    signal sig_layer4_karnel_out : std_logic := '0';
    signal sig_delay_layer4_karnel_out : unsigned(9 downto 0) := (others => '0');
    --for affine data--
    signal uni_data_layer_affine_a : signed(data_len*36 -1 downto 0) := (others => '0');
    signal uni_data_layer_affine_b : signed(data_len*36 -1 downto 0) := (others => '0');
    signal sig_layer_affine_36_data_out : std_logic := '0';
    signal sig_layer_affine_out : std_logic := '0';
    signal sig_delay_layer_affine_out : unsigned(13 downto 0) := (others => '0');
    --for comparetor--
    signal affine_out : signed(data_len*12 -1 downto 0) := (others => '0');
    --for cube_data--
    signal cube_data_temp : signed(data_len*48 -1 downto 0) := (others => '0');
    signal cube_data_delay : signed(data_len*48 -1 downto 0) := (others => '0');
    --store data function--
    function store_cube_data (in_data : unsigned(99 downto 0) := (others => '0')) return signed is
        variable return_temp : signed(data_len*48 -1 downto 0) := (others => '0');
    begin
        for corner_num in 0 to 3 loop
            return_temp(return_temp'length-data_len*corner_num-(data_int-3) -1 downto return_temp'length-data_len*corner_num-data_int) := signed(in_data(in_data'length-5*corner_num -1 downto in_data'length-5*corner_num-3));--0~3:corner_num store
            return_temp(return_temp'length-data_len*(12+corner_num)-(data_int-2) -1 downto return_temp'length-data_len*(12+corner_num)-data_int) := signed(in_data(in_data'length-5*corner_num-3 -1 downto in_data'length-5*(corner_num+1)));--0~3:corner_dir store
            return_temp(return_temp'length-data_len*(corner_num+8)-(data_int-3) -1 downto return_temp'length-data_len*(corner_num+8)-data_int) := signed(in_data(in_data'length-5*(corner_num+4) -1 downto in_data'length-5*(corner_num+4)-3));--4~7:corner_num store
            return_temp(return_temp'length-data_len*(12+corner_num+8)-(data_int-2) -1 downto return_temp'length-data_len*(12+corner_num+8)-data_int) := signed(in_data(in_data'length-5*(corner_num+4)-3 -1 downto in_data'length-5*(corner_num+4+1)));--0~3:corner_dir store
        end loop;
        for edge_num in 0 to 11 loop
            return_temp(return_temp'length-data_len*(24+edge_num)-(data_int-4) -1 downto return_temp'length-data_len*(24+edge_num)-data_int) := signed(in_data(in_data'length-5*(8+edge_num) -1 downto in_data'length-5*(8+edge_num)-4));
            return_temp(return_temp'length-data_len*(36+edge_num)-(data_int-1) -1 downto return_temp'length-data_len*(36+edge_num)-data_int) := signed(in_data(in_data'length-5*(8+edge_num)-4 -1 downto in_data'length-5*(8+edge_num+1)));
        end loop;
        
        return return_temp;
    end store_cube_data;
begin

    --counter522--
    layer1_counter : counter_layer1
        port map (
        --input--
            clk => clk,
            reset => reset_layer1,
        --output--
            count => count_layer1
        );
    
    --counter3501--
    layer2_counter : counter_layer2
        port map (
        --input--
            clk => clk,
            reset => reset_layer2,
        --output--
            count => count_layer2
        );
    
    --counter3501--
    layer3_counter : counter_layer2
        port map (
        --input--
            clk => clk,
            reset => reset_layer3,
        --output--
            count => count_layer3
        );
    
    --counter3501--
    layer4_counter : counter_layer2
        port map (
        --input--
            clk => clk,
            reset => reset_layer4,
        --output--
            count => count_layer4
        );

    --counter206--
    layer_affine_counter : counter_layer_affine
        port map (
        --input--
            clk => clk,
            reset => reset_layer_affine,
        --output--
            count => count_layer_affine
        );

    --zero padding--
    --padding initial parameter--
    zeropadding_0 : padding_data_re
        generic map (
            data_int => data_int,
            data_dec => data_dec
        )
        port map (
        --input--
            clk => clk,
            reset => reset_layer1,
            count => count_layer1,
            in_data => cube_data_delay(cube_data_delay'length -1 downto cube_data_delay'length-data_len),
        --output--
            sig_36_data_out => sig_layer1_karnel_out,
            uni_data => uni_karnel_data_layer1
        );
    
    --padding calc out layer1--
    zeropadding_1 : padding_calc_out_1_re
        generic map (
            data_int => data_int,
            data_dec => data_dec
        )
        port map (
        --input--
            clk => clk,
            reset_layer1 => reset_layer1,
            sig_calc_end => sig_delay_layer1_karnel_out(9),
            reset_layer2 => reset_layer2,
            count_layer2 => count_layer2,
            in_data => eLU_out,
        --output--
            sig_36_data_out => sig_layer2_36_data_out,
            sig_uni_karnel_out => sig_layer2_karnel_out,
            uni_data => uni_karnel_data_layer2
        );
        
    --padding calc out layer2--
    zeropadding_2 : padding_calc_out_2_re
        generic map (
            data_int => data_int,
            data_dec => data_dec
        )
        Port map (
        --input--
            clk => clk,
            reset_layer2 => reset_layer2,
            sig_calc_out => sig_delay_layer2_karnel_out(9),
            reset_layer3 => reset_layer3,
            count_layer3 => count_layer3,
            in_data => eLU_out,
        --output--
            sig_36_data_out => sig_layer3_36_data_out,
            sig_uni_karnel_out => sig_layer3_karnel_out,
            uni_data => uni_karnel_data_layer3
        );
    
    --padding calc out layer3--
    zeropadding_3 : padding_calc_out_2_re --layer2̎g܂킵
        generic map (
            data_int => data_int,
            data_dec => data_dec
        )
        Port map (
        --input--
            clk => clk,
            reset_layer2 => reset_layer3,
            sig_calc_out => sig_delay_layer3_karnel_out(9),
            reset_layer3 => reset_layer4,
            count_layer3 => count_layer4,
            in_data => eLU_out,
        --output--
            sig_36_data_out => sig_layer4_36_data_out,
            sig_uni_karnel_out => sig_layer4_karnel_out,
            uni_data => uni_karnel_data_layer4
        );
    
    --store calc out layer4--
    affine_data : store_layer4_re
        generic map (
            data_int => data_int,
            data_dec => data_dec
        )
        port map (
        --input--
            clk => clk,
            reset_layer4 => reset_layer4,
            sig_calc_out => sig_delay_layer4_karnel_out(9),
            reset_layer_affine => reset_layer_affine,
            count_layer_affine => count_layer_affine,
            in_data => eLU_out,
        --output--
            sig_36_data_out => sig_layer_affine_36_data_out,
            sig_uni_data_out => sig_layer_affine_out,
            uni_data_a => uni_data_layer_affine_a,
            uni_data_b => uni_data_layer_affine_b
        );

    --compare q_value--
    comparetor : compare_q_value
        generic map (
            data_len => data_len
        )
        port map (
        --input--
            clk => clk,
            sig_in => sig_delay_layer_affine_out(6),
            in_data => affine_out,
        --output--
            move_num => move_num
        );
    
    --gen weight addr--
    w_addr : gen_w_addr
        port map (
        --input--
            clk => clk,
            reset_layer1 => reset_layer1,
            reset_layer2 => reset_layer2,
            reset_layer3 => reset_layer3,
            reset_layer4 => reset_layer4,
            reset_layer_affine => reset_layer_affine,
            count_layer1 => count_layer1,
            count_layer2 => count_layer2,
            count_layer3 => count_layer3,
            count_layer4 => count_layer4,
            count_layer_affine => count_layer_affine,
        --output--
            bias_rom_en => bias_rom_en,
            bias_addr => bias_addr,
            w_rom_en => w_rom_en,
            w_addr => w_rom_addr
        );
        
    --weight data:layer_1--
    weight_data : for ch_num in 0 to 31 generate
    w_unit : w_rom
        generic map (
            file_name => "weight_ch_" & integer'image(ch_num) & ".txt", --integer'image:𕶎ɕϊ
            --file_name => "w_data_test.txt",
            data_len => w_len,
            addr_bit => w_rom_addr_bit
        )
        port map (
        --input--
            clk => clk,
            ena => w_rom_en(ch_num),
            addr => w_rom_addr,
        --output--
            dout => w_rom_data(ch_num)
        );
    end generate weight_data;
    
    --bias data--
    bias : bias_rom
        generic map (
            file_name => "bias_data.txt",
            data_len => w_len,
            addr_bit => bias_addr_bit
        )
        port map (
        --input--
            clk => clk,
            ena => bias_rom_en,
            addr => bias_addr,
        --output--
            dout => bias_temp
        );

    
    --calculation unit--
    --cnn--
    calc_cnn_ch : for ch_num in 0 to 31 generate
    cnn : calc_cnn
        generic map (
            in_int => data_int,
            in_dec => data_dec,
            w_int => w_int,
            w_dec => w_dec
        )
        port map (
        --input--
            clk => clk,
            ch_data => uni_karnel_data(ch_num),
            ch_w => uni_karnel_w(ch_num),
            ch_bias => bias_data(ch_num),
        --output--
            ch_out => calc_out(ch_num)
        );   
    end generate calc_cnn_ch;
    
    --eLU_function--
    eLU : eLU_function
    generic map (
        data_int => data_int,
        data_dec => data_dec
    )
    port map (
    --input--
        clk => clk,
        eLU_in => calc_out_delay(0), --(1,7,16)--
    --output--
        eLU_out => eLU_out --(1,7,16)--
    );
    
    
    
    --function--
    --cube_data store--
    cube_data_temp <= store_cube_data(cube_data);
    process(clk)
    begin
        if (clk'event and clk='1') then
            if (to_integer(count_layer1) = 0 and reset_layer1 = '0') then
                cube_data_delay <= cube_data_temp;
            else
                for i in 0 to 46 loop
                    cube_data_delay(cube_data_delay'length-data_len*i -1 downto cube_data_delay'length-data_len*(i+1)) <= cube_data_delay(cube_data_delay'length-data_len*(i+1) -1 downto cube_data_delay'length-data_len*(i+2));
                end loop;
                cube_data_delay(cube_data_delay'length-data_len*47 -1 downto 0) <= (others => '0');
            end if;
        end if;
    end process;
    --sig_network_out--
    sig_move_out <= sig_delay_layer_affine_out(12);
    
    --controll layer counter--
    process(clk)
    begin
        if (clk'event and clk='1') then
            if (reset = '1') then
                reset_layer1 <= '1';
                reset_layer2 <= '1';
                reset_layer3 <= '1';
                reset_layer4 <= '1';
                reset_layer_affine <= '1';
            else
                --start layer2--
                if (to_integer(count_layer1) = 521) then
                    reset_layer1 <= '1';
                    reset_layer2 <= '0';
                --start layer3--
                elsif (to_integer(count_layer2) = 3500) then
                    reset_layer2 <= '1';
                    reset_layer3 <= '0';
                --start layer4--
                elsif (to_integer(count_layer3) = 3500) then
                    reset_layer3 <= '1';
                    reset_layer4 <= '0';
                --start layer_affine--
                elsif (to_integer(count_layer4) = 3500) then
                    reset_layer4 <= '1';
                    reset_layer_affine <= '0';
                --end layer_affine--
                elsif (to_integer(count_layer_affine) = 205) then
                    reset_layer_affine <= '1';
                --start layer1--
                elsif (reset = '0' and reset_layer2 = '1' and reset_layer3 = '1' and reset_layer4 = '1' and reset_layer_affine = '1' and reset_layer_affine = '1') then
                    reset_layer1 <= '0';
                end if;
            end if;
        end if;
    end process;

    --uni_karnel_w--
    process(clk)
    begin
        if (clk'event and clk='1') then
            if (reset = '1') then
                uni_karnel_w_temp <= (others => (others => '0'));
            --for calc 1st layer
            elsif (reset_layer1 = '0') then
                if (to_integer(count_layer1) > 49 and to_integer(count_layer1) < 86) then --padding dataǂݍ݂̒xl49~86
                    for ch_num in 0 to 31 loop
                        uni_karnel_w_temp(ch_num)(uni_karnel_w_temp(ch_num)'length-data_len*35 -1 downto 0) <= w_rom_data(ch_num);
                        uni_karnel_w_temp(ch_num)(uni_karnel_w_temp(ch_num)'length -1 downto data_len) <= uni_karnel_w_temp(ch_num)(uni_karnel_w_temp(ch_num)'length-data_len -1 downto 0);
                    end loop;
                end if;
            --for calc 2nd layer
            elsif (reset_layer2 = '0') then
                for ch_num in 0 to 31 loop
                    uni_karnel_w_temp(ch_num)(uni_karnel_w_temp(ch_num)'length-data_len*35 -1 downto 0) <= w_rom_data(ch_num);
                    uni_karnel_w_temp(ch_num)(uni_karnel_w_temp(ch_num)'length -1 downto data_len) <= uni_karnel_w_temp(ch_num)(uni_karnel_w_temp(ch_num)'length-data_len -1 downto 0);
                end loop;
            --for calc 3rd layer
            elsif (reset_layer3 = '0') then
                for ch_num in 0 to 31 loop
                    uni_karnel_w_temp(ch_num)(uni_karnel_w_temp(ch_num)'length-data_len*35 -1 downto 0) <= w_rom_data(ch_num);
                    uni_karnel_w_temp(ch_num)(uni_karnel_w_temp(ch_num)'length -1 downto data_len) <= uni_karnel_w_temp(ch_num)(uni_karnel_w_temp(ch_num)'length-data_len -1 downto 0);
                end loop;
            --for calc 4th layer
            elsif (reset_layer4 = '0') then
                for ch_num in 0 to 31 loop
                    uni_karnel_w_temp(ch_num)(uni_karnel_w_temp(ch_num)'length-data_len*35 -1 downto 0) <= w_rom_data(ch_num);
                    uni_karnel_w_temp(ch_num)(uni_karnel_w_temp(ch_num)'length -1 downto data_len) <= uni_karnel_w_temp(ch_num)(uni_karnel_w_temp(ch_num)'length-data_len -1 downto 0);
                end loop;
            --for calc affine layer
            elsif (reset_layer_affine = '0') then
                for ch_num in 0 to 31 loop
                    uni_karnel_w_temp(ch_num)(uni_karnel_w_temp(ch_num)'length-data_len*35 -1 downto 0) <= w_rom_data(ch_num);
                    uni_karnel_w_temp(ch_num)(uni_karnel_w_temp(ch_num)'length -1 downto data_len) <= uni_karnel_w_temp(ch_num)(uni_karnel_w_temp(ch_num)'length-data_len -1 downto 0);
                end loop;
            end if;
        end if;
    end process;
    
    --uni_karnel_w out timing--
    process(clk)
    begin
        if (clk'event and clk='1') then
            if (reset_layer1 = '0') then
                if (sig_delay_layer1_karnel_out(1) = '1') then
                    uni_karnel_w <= uni_karnel_w_temp;
                else
                    uni_karnel_w <= (others => (others => '0'));
                end if;
            elsif (reset_layer2 = '0') then
                if (sig_layer2_36_data_out = '1') then
                    uni_karnel_w <= uni_karnel_w_temp;
                else
                    uni_karnel_w <= (others => (others => '0'));
                end if;
            elsif (reset_layer3 = '0') then
                if (sig_layer3_36_data_out = '1') then
                    uni_karnel_w <= uni_karnel_w_temp;
                else
                    uni_karnel_w <= (others => (others => '0'));
                end if;
            elsif (reset_layer4 = '0') then
                if (sig_layer4_36_data_out = '1') then
                    uni_karnel_w <= uni_karnel_w_temp;
                else
                    uni_karnel_w <= (others => (others => '0'));
                end if;
            elsif (reset_layer_affine = '0') then
                if (to_integer(count_layer_affine) = 194) then
                    for ch_num in 0 to 23 loop
--                        uni_karnel_w(ch_num)(uni_karnel_w(ch_num)'length -1 downto uni_karnel_w(ch_num)'length-data_len*12) <= uni_karnel_w_temp(ch_num)(uni_karnel_w_temp(ch_num)'length -1 downto uni_karnel_w_temp(ch_num)'length-data_len*12); 
                        uni_karnel_w(ch_num)(uni_karnel_w(ch_num)'length -1 downto uni_karnel_w(ch_num)'length-data_len*12) <= uni_karnel_w_temp(ch_num)(data_len*12 -1 downto 0);
                    end loop;
                elsif (sig_layer_affine_36_data_out = '1') then
                    for ch_num in 0 to 23 loop
                        uni_karnel_w(ch_num) <= uni_karnel_w_temp(ch_num);
                    end loop;
                else
                    uni_karnel_w <= (others => (others => '0'));
                end if;
            end if;
        end if;
    end process;
    
    --bias_data--
    --bias out--
    process(clk)
    begin
        if (clk'event and clk='1') then
            if (reset = '1') then
                bias_data_temp <= (others => (others => '0'));
            --for 1st layer
            elsif (reset_layer1 = '0') then
                if (to_integer(count_layer1) > 49 and to_integer(count_layer1) <= 82) then --padding dataǂݍ݂̒xl49~82
                    if (to_integer(bias_addr_delay) < 32) then
                        bias_data_temp(to_integer(bias_addr_delay)) <= bias_temp;
                    end if;
                end if;
            --for 2nd layer
            elsif (reset_layer2 = '0') then
                if (to_integer(bias_addr_delay) > 0) then
                    bias_data_temp(to_integer(bias_addr_delay)-32) <= bias_temp;
                end if;
            --for 3rd layer
            elsif (reset_layer3 = '0') then
                if (to_integer(bias_addr_delay) > 0) then
                    bias_data_temp(to_integer(bias_addr_delay)-64) <= bias_temp;
                end if;
            --for 4th layer
            elsif (reset_layer4 = '0') then
                if (to_integer(bias_addr_delay) > 0) then
                    bias_data_temp(to_integer(bias_addr_delay)-96) <= bias_temp;
                end if;
            --for affine layer
            elsif (reset_layer_affine = '0') then
                if (to_integer(bias_addr_delay) > 0) then
                    bias_data_temp(to_integer(bias_addr_delay)+12-128) <= bias_temp;--㔼12~23chɑ
                end if;
            end if;
        end if;
    end process;
    
    --bias_data out timing--
    process(clk)
    begin
        if (clk'event and clk='1') then
            if (reset_layer1 = '0') then
                if (sig_delay_layer1_karnel_out(1) = '1') then
                    bias_data <= bias_data_temp;
                else
                    bias_data <= (others => (others => '0'));
                end if;
            elsif (reset_layer2 = '0') then
                if (sig_delay_layer2_karnel_out(0) = '1') then
                    bias_data <= bias_data_temp;
                else
                    bias_data <= (others => (others => '0'));
                end if;
            elsif (reset_layer3 = '0') then
                if (sig_delay_layer3_karnel_out(0) = '1') then
                    bias_data <= bias_data_temp;
                else
                    bias_data <= (others => (others => '0'));
                end if;
            elsif (reset_layer4 = '0') then
                if (sig_delay_layer4_karnel_out(0) = '1') then
                    bias_data <= bias_data_temp;
                else
                    bias_data <= (others => (others => '0'));
                end if;
            elsif (reset_layer_affine = '0') then
                if (sig_delay_layer_affine_out(0) = '1') then
                    for ch_num in 12 to 23 loop
                        bias_data(ch_num) <= bias_data_temp(ch_num);
                    end loop;
                else
                    bias_data <= (others => (others => '0'));
                end if;
            end if;
        end if;
    end process;
    
    --for data_in to cnn_unit--
    process(uni_karnel_data_layer1, uni_karnel_data_layer2, uni_karnel_data_layer3, uni_karnel_data_layer4, uni_data_layer_affine_a, uni_data_layer_affine_b)
    begin
        if (reset_layer1 = '0') then
            for ch_num in 0 to 31 loop
                uni_karnel_data(ch_num) <= uni_karnel_data_layer1;
            end loop;
        elsif (reset_layer2 = '0') then
            for ch_num in 0 to 31 loop
                uni_karnel_data(ch_num) <= uni_karnel_data_layer2;
            end loop;
        elsif (reset_layer3 = '0') then
            for ch_num in 0 to 31 loop
                uni_karnel_data(ch_num) <= uni_karnel_data_layer3;
            end loop;
        elsif (reset_layer4 = '0') then
            for ch_num in 0 to 31 loop
                uni_karnel_data(ch_num) <= uni_karnel_data_layer4;
            end loop;
        elsif (reset_layer_affine = '0') then
            for ch_num in 0 to 11 loop
                uni_karnel_data(ch_num) <= uni_data_layer_affine_a;--Of[^
                uni_karnel_data(ch_num+12) <= uni_data_layer_affine_b;--㔼f[^
            end loop;
        else
            uni_karnel_data <= (others => (others => '0'));
        end if;
    end process;

    --calc_out_delay timing--
    process(clk)
    begin
        if (clk'event and clk='1') then
            if (reset = '1') then
                calc_out_delay <= (others => (others => '0'));
            --layer1--
            elsif (reset_layer1 = '0') then
                if (sig_delay_layer1_karnel_out(6) = '1') then
                    calc_out_delay <= calc_out;
                else
                --for calc_out_delay--
                    for ch_num in 0 to 30 loop
                        calc_out_delay(ch_num) <= calc_out_delay(ch_num+1);
                    end loop;
                    calc_out_delay(31) <= (others => '0');
                end if;
            --layer2--
            elsif (reset_layer2 = '0') then
                if (sig_delay_layer2_karnel_out(6) = '1') then
                    calc_out_delay <= add_ch_out;
                    add_ch_out <= (others => (others => '0'));
                else
                --add 8 ch_out data--
                    for ch_num in 0 to 31 loop
                        add_ch_out(ch_num) <= add_ch_out(ch_num) + calc_out(ch_num);
                    end loop;
                --for calc_out_delay--
                    for ch_num in 0 to 30 loop
                        calc_out_delay(ch_num) <= calc_out_delay(ch_num+1);
                    end loop;
                    calc_out_delay(31) <= (others => '0');
                end if;
            --layer3--
            elsif (reset_layer3 = '0') then
                if (sig_delay_layer3_karnel_out(6) = '1') then
                    calc_out_delay <= add_ch_out;
                    add_ch_out <= (others => (others => '0'));
                else
                --add 8 ch_out data--
                    for ch_num in 0 to 31 loop
                        add_ch_out(ch_num) <= add_ch_out(ch_num) + calc_out(ch_num);
                    end loop;
                --for calc_out_delay--
                    for ch_num in 0 to 30 loop
                        calc_out_delay(ch_num) <= calc_out_delay(ch_num+1);
                    end loop;
                    calc_out_delay(31) <= (others => '0');
                end if;
            --layer4--
            elsif (reset_layer4 = '0') then
                if (sig_delay_layer4_karnel_out(6) = '1') then
                    calc_out_delay <= add_ch_out;
                    add_ch_out <= (others => (others => '0'));
                else
                --add 8 ch_out data--
                    for ch_num in 0 to 31 loop
                        add_ch_out(ch_num) <= add_ch_out(ch_num) + calc_out(ch_num);
                    end loop;
                --for calc_out_delay--
                    for ch_num in 0 to 30 loop
                        calc_out_delay(ch_num) <= calc_out_delay(ch_num+1);
                    end loop;
                    calc_out_delay(31) <= (others => '0');
                end if;
            --layer_affine--
            elsif (reset_layer_affine = '0') then
                if (sig_delay_layer_affine_out(6) = '1') then
                    for ch_num in 0 to 11 loop
                        affine_out(affine_out'length-data_len*ch_num -1 downto affine_out'length-data_len*(ch_num+1)) <= add_ch_out(ch_num) + add_ch_out(ch_num+12);
                    end loop;
                    add_ch_out <= (others => (others => '0'));
                elsif (sig_delay_layer_affine_out(7) = '1') then
                    affine_out <= (others => '0');
                else
                --add 8 ch_out data--
                    for ch_num in 0 to 23 loop
                        add_ch_out(ch_num) <= add_ch_out(ch_num) + calc_out(ch_num);
                    end loop;
                end if;
            end if;
        end if;
    end process;
    
    --DFF--
    process(clk)
    begin
        if (clk'event and clk='1') then
        --for w_out_timing--
            w_rom_addr_delay <= w_rom_addr;
        --for bias out/weight out/calc out timing : layer_1̃pfBO36f[^ǂݏo9clockx--
            sig_delay_layer1_karnel_out(0) <= sig_layer1_karnel_out;
            for i in 0 to 8 loop
                sig_delay_layer1_karnel_out(i+1) <= sig_delay_layer1_karnel_out(i);
            end loop;
        --for bias out/weight out/calc out timing : layer2--
            bias_addr_delay <= bias_addr;
            sig_delay_layer2_karnel_out(0) <= sig_layer2_karnel_out;
            for i in 0 to 8 loop
                sig_delay_layer2_karnel_out(i+1) <= sig_delay_layer2_karnel_out(i);
            end loop;
        --for bias out/weight out/calc out timing : layer3--
            sig_delay_layer3_karnel_out(0) <= sig_layer3_karnel_out;
            for i in 0 to 8 loop
                sig_delay_layer3_karnel_out(i+1) <= sig_delay_layer3_karnel_out(i);
            end loop;
        --for bias out/weight out/calc out timing : layer4--
            sig_delay_layer4_karnel_out(0) <= sig_layer4_karnel_out;
            for i in 0 to 8 loop
                sig_delay_layer4_karnel_out(i+1) <= sig_delay_layer4_karnel_out(i);
            end loop;
        --for bias out/weight out/calc out timing : layer_affine--
            sig_delay_layer_affine_out(0) <= sig_layer_affine_out;
            for i in 0 to 11 loop
                sig_delay_layer_affine_out(i+1) <= sig_delay_layer_affine_out(i);
            end loop;
        end if;
    end process;
    
end Behavioral;