----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date: 2022/01/14 16:35:13
-- Design Name: 
-- Module Name: main_sub - Behavioral
-- Project Name: 
-- Target Devices: 
-- Tool Versions: 
-- Description: 
-- 
-- Dependencies: 
-- 
-- Revision:
-- Revision 0.01 - File Created
-- Additional Comments:
-- check (1,7,16)(1,6,10)
----------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use ieee.numeric_std.all;

-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
--use IEEE.NUMERIC_STD.ALL;

-- Uncomment the following library declaration if instantiating
-- any Xilinx leaf cells in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity main_sub is
    Port (
    --input--
        clk : in STD_LOGIC;
        reset : in std_logic;
        sig_data_in : in std_logic;
        cube_data : in unsigned(99 downto 0);
    --output--
        sig_solve_end : out std_logic;
        sig_solve_step : out unsigned(3 downto 0) 
    );
end main_sub;

architecture Behavioral of main_sub is

    component forward_re
        Port (
        --input--
            clk : in STD_LOGIC;
            reset : in std_logic;
            cube_data : in unsigned(99 downto 0);
        --output--
            sig_move_out : out std_logic;
            move_num : out unsigned(3 downto 0)
        );
    end component;
    
    component cube_model
        Port (
        --input--
            clk : in STD_LOGIC;
            reset : in std_logic;
            sig_change_cube : in std_logic;
            move_step : in unsigned(3 downto 0);
            current_cube : in unsigned(99 downto 0);
        --output--
            sig_corr_cube : out std_logic := '0';
            sig_update_cube : out std_logic := '0';
            update_cube : out unsigned(99 downto 0)
        );
    end component;
    
    signal current_cube : unsigned(99 downto 0) := (others => '0');
    signal model_cube_in : unsigned(99 downto 0) := (others => '0');
    signal update_cube : unsigned(99 downto 0) := (others => '0');
    signal model_reset : std_logic := '0';
    signal sig_change_cube : std_logic := '0';
    signal sig_corr_cube : std_logic := '0';
    signal sig_update_cube : std_logic := '0';
    signal next_move : unsigned(3 downto 0) := (others => '0');
    
    signal forward_cube_in : unsigned(99 downto 0) := (others => '0');
    signal reset_forward : std_logic := '1';
    signal sig_move_out : std_logic := '0';
    signal move_step : unsigned(3 downto 0) := (others => '0');
    signal sig_solve_end_temp : std_logic := '0';
    signal count_step : unsigned(5 downto 0) := (others => '0');
begin

    --cube_model--
    rubiks_cube : cube_model
        port map (
        --input--
            clk => clk,
            reset => sig_data_in,
            sig_change_cube => sig_change_cube,
            move_step => next_move,
            current_cube => model_cube_in,
        --output--
            sig_corr_cube => sig_corr_cube,
            sig_update_cube => sig_update_cube,
            update_cube => update_cube
        );

    --forward process--
    forward_module : forward_re
        port map (
        --input--
            clk => clk,
            reset => reset_forward,
            cube_data => forward_cube_in,
        --output--
            sig_move_out => sig_move_out,
            move_num => move_step
        );
        
    --function--
    --controll forward module--
    process(clk)
    begin
        if (clk'event and clk='1') then
            if (sig_data_in = '1') then
                if (reset_forward = '1') then
                    reset_forward <= '0';
                    forward_cube_in <= cube_data;
                end if;
            elsif (sig_solve_end_temp ='1') then
                reset_forward <= '1';
            elsif (sig_update_cube = '1' and to_integer(count_step) /= 50) then
                if (reset_forward = '1') then
                    reset_forward <= '0';
                    forward_cube_in <= update_cube;
                end if;
            elsif (sig_update_cube = '1' and to_integer(count_step) = 50) then
                reset_forward <= '1';
                forward_cube_in <= (others => '0');
            elsif (sig_move_out = '1') then
                if (reset_forward = '0') then
                    reset_forward <= '1';
                    forward_cube_in <= (others => '0');
                end if;
            end if;
        end if;
    end process;
    
    --count_step--
    process(clk)
    begin
        if (clk'event and clk='1') then
            if (sig_corr_cube = '1' or sig_data_in = '1') then --烊Zbg
                count_step <= (others => '0');
            elsif (to_integer(count_step) = 50 and sig_update_cube = '1') then --_50ڂsig_update_cubeŋIɏI
                count_step <= (others => '0');
            elsif (sig_move_out = '1') then --sig_move_outŐ_񐔃JEg
                count_step <= count_step + to_unsigned(1, count_step'length);
            end if;
        end if;
    end process;
    
    --current cube_data--
    process(clk)
    begin
        if (clk'event and clk='1') then
            if (sig_data_in = '1') then
                current_cube <= cube_data;
            elsif (sig_update_cube = '1') then
                current_cube <= update_cube;
            end if;
        end if;
    end process;
    
    --model in/out--
    process(clk)
    begin
        if (clk'event and clk='1') then
            if (sig_move_out = '1') then
                model_cube_in <= current_cube;
                sig_change_cube <= sig_move_out;
                next_move <= move_step;
            else
                sig_change_cube <= sig_move_out;
                next_move <= move_step;
            end if;
        end if;
    end process;
    
    --sig_solve_end/sig_solve_step--
    process(clk)
    begin
        if (clk'event and clk='1') then
            if (sig_data_in = '1') then
                sig_solve_end_temp <= '0';
            elsif (sig_corr_cube = '1') then
                sig_solve_end_temp <= '1';
                sig_solve_step <= move_step;
            elsif (sig_update_cube = '1' and to_integer(count_step) = 50) then
                sig_solve_end_temp <= '1';
                sig_solve_step <= (others => '1');--I"1111"=15o
            else
                sig_solve_step <= move_step;
            end if;
        end if;
    end process;
    sig_solve_end <= sig_solve_end_temp;
    
end Behavioral;
