----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date: 2022/01/14 17:36:40
-- Design Name: 
-- Module Name: padding_calc_out_1_re - Behavioral
-- Project Name: 
-- Target Devices: 
-- Tool Versions: 
-- Description: 
-- 
-- Dependencies: 
-- 
-- Revision:
-- Revision 0.01 - File Created
-- Additional Comments:
-- 
----------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use ieee.numeric_std.all;

-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
--use IEEE.NUMERIC_STD.ALL;

-- Uncomment the following library declaration if instantiating
-- any Xilinx leaf cells in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity padding_calc_out_1_re is
    generic (
        data_int : integer;
        data_dec : integer
    );
    Port (
    --input--
        clk : in STD_LOGIC;
        reset_layer1 : in std_logic;
        sig_calc_end : in std_logic;
        reset_layer2 : in std_logic;
        count_layer2 : in unsigned(11 downto 0);
        in_data : in signed((data_int+data_dec) -1 downto 0);
    --output--
        sig_36_data_out : out std_logic;
        sig_uni_karnel_out : out std_logic;
        uni_data : out signed((data_int+data_dec)*36 -1 downto 0)
    );
end padding_calc_out_1_re;

architecture Behavioral of padding_calc_out_1_re is
    --conv addr--
    --layer_1 out padding addr--
    component conv_padding_addr_layer2_re
        Port (
        --input--
        clk : in STD_LOGIC;
        reset : in std_logic;
        sig_calc_end : in std_logic; 
        --output--
        ram_wea : out std_logic := '0';
        addr : out unsigned(9 downto 0)
        );
    end component;
    
    --layer_2 karnel addr--
    component conv_layer_2_karnel_addr_re
    Port (
    --input--
        clk : in STD_LOGIC;
        reset : in std_logic;
        count : in unsigned(11 downto 0);
    --output--
        sig_36_data_out : out std_logic;
        sig_uni_karnel_out : out std_logic;
--        count36 : out unsigned(5 downto 0);
        addr : out unsigned(9 downto 0) --960<1024=2^10;
    );
    end component;
    
    
    --store ram--
    component padding_ram_layer_1_out
        generic (
            data_bit : integer;
            addr_bit : integer --960 < 1024 = 2^10
        );
        Port ( 
        --input--
            clk : in STD_LOGIC;
            ena : in std_logic;
            wea : in std_logic;
            addra : in unsigned(addr_bit -1 downto 0);
            addrb : in unsigned(addr_bit -1 downto 0);
            dina : in signed(data_bit -1 downto 0);
        --output--
            doutb : out signed(data_bit -1 downto 0)
        );
    end component;


    --internal signal--
    constant data_len : integer := data_int+data_dec;
    constant addr_bit : integer := 10;
    
    signal ram_wea : std_logic := '0';
    signal ram_ena : std_logic := '0';
    signal padding_addr : unsigned(addr_bit -1 downto 0) := (others => '0');
    signal read_addr : unsigned(addr_bit -1 downto 0) := (others => '0');
    signal delay_reset_layer1 : std_logic := '0';
    signal data_out_temp : signed(data_len -1 downto 0) := (others => '0');
    signal delay_reset_layer2 : std_logic := '0';
    signal delay_sig_36_data_out : unsigned(1 downto 0) := (others => '0');
    signal count36, count36_delay : unsigned(5 downto 0) := (others => '0');
    signal uni_data_temp : signed(data_len*36 -1 downto 0) := (others => '0');
begin

    conv_padding_addr : conv_padding_addr_layer2_re
        port map (
        --input--
            clk => clk,
            reset => delay_reset_layer1,
            sig_calc_end => sig_calc_end,
        --output--
            ram_wea => ram_wea,
            addr => padding_addr
        );
    
    conv_karnel_addr : conv_layer_2_karnel_addr_re
        port map (
        --input--
            clk => clk,
            reset => reset_layer2,
            count => count_layer2,
        --output--
            sig_36_data_out => delay_sig_36_data_out(0),
            sig_uni_karnel_out => sig_uni_karnel_out,
--            count36 => count36,
            addr => read_addr
        );
    
    
    padding_ram : padding_ram_layer_1_out
        generic map (
            data_bit => data_len,
            addr_bit => addr_bit
        )
        port map (
        --input--
            clk => clk,
            ena => ram_ena,
            wea => ram_wea,
            addra => padding_addr,
            addrb => read_addr,
            dina => in_data,
        --output--
            doutb => data_out_temp
        );
    
    --function--
    ram_ena <= not delay_reset_layer2;
    
    --batch data--
    process(clk)
    begin
        if (clk'event and clk='1') then
            if (reset_layer2 = '1') then
                uni_data_temp <= (others => '0');
            else
--                uni_data_temp(uni_data_temp'length- to_integer(count36_delay)*data_len -1 downto uni_data_temp'length- (to_integer(count36_delay)+1)*data_len) <= data_out_temp;
                uni_data_temp(uni_data_temp'length-35*data_len -1 downto 0) <= data_out_temp;
                uni_data_temp(uni_data_temp'length -1 downto data_len) <= uni_data_temp(uni_data_temp'length-data_len -1 downto 0);
            end if;
        end if;
    end process;
    
    process(clk)
    begin
        if (clk'event and clk='1') then
            if (delay_sig_36_data_out(1) = '1') then
                uni_data <= uni_data_temp;
            else
                uni_data <= (others => '0');
            end if;
        end if;
    end process;
    sig_36_data_out <= delay_sig_36_data_out(1);
    
    --DFF--
    process(clk)
    begin
        if (clk'event and clk='1') then
            delay_reset_layer1 <= reset_layer1; --ram_weâ߂ɒx点ȂƍŌ̃f[^ĂȂ
            delay_reset_layer2 <= reset_layer2;
--            count36_delay <= count36;
            delay_sig_36_data_out(1) <= delay_sig_36_data_out(0);
        end if;
    end process;
    

end Behavioral;