----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date: 2022/01/16 10:55:15
-- Design Name: 
-- Module Name: store_layer4_re - Behavioral
-- Project Name: 
-- Target Devices: 
-- Tool Versions: 
-- Description: 
-- 
-- Dependencies: 
-- 
-- Revision:
-- Revision 0.01 - File Created
-- Additional Comments:
-- 
----------------------------------------------------------------------------------


library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use ieee.numeric_std.all;

-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
--use IEEE.NUMERIC_STD.ALL;

-- Uncomment the following library declaration if instantiating
-- any Xilinx leaf cells in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity store_layer4_re is
    generic (
        data_int : integer;
        data_dec : integer
    );
    Port (
    --input--
        clk : in STD_LOGIC;
        reset_layer4 : in std_logic;
        sig_calc_out : in std_logic;
        reset_layer_affine : in std_logic;
        count_layer_affine : in unsigned(7 downto 0);
        in_data : in signed((data_int+data_dec) -1 downto 0);
    --output--
        sig_36_data_out : out std_logic;
        sig_uni_data_out : out std_logic;
        uni_data_a : out signed((data_int+data_dec)*36 -1 downto 0);
        uni_data_b : out signed((data_int+data_dec)*36 -1 downto 0)
    );
end store_layer4_re;

architecture Behavioral of store_layer4_re is
    
    --gen store_addr layer4_out data--
    component conv_store_addr_layer4_out
        Port (
        --input--
            clk : in STD_LOGIC;
            reset : in std_logic;
            sig_calc_out : in std_logic;
        --output--
            ram_wea_a : out std_logic;
            ram_wea_b : out std_logic;
            addr : out unsigned(7 downto 0)
        );
    end component;
    
    --gen read addr--
    component conv_layer_affine_read_addr_re
        Port (
        --input--
            clk : in std_logic;
            reset : in std_logic;
            count : in unsigned(7 downto 0);
        --output--
            sig_36_data_out : out std_logic;
            sig_uni_data_out : out std_logic := '0';
            addr : out unsigned(7 downto 0) --192<256=2^8;
        );
    end component;
    
    --store_ram--
    component store_layer4_out_ram
        generic (
            data_bit : integer;
            addr_bit : integer --960 < 1024 = 2^10
        );
        Port ( 
        --input--
            clk : in STD_LOGIC;
            ena : in std_logic;
            wea : in std_logic;
            addra : in unsigned(addr_bit -1 downto 0);
            addrb : in unsigned(addr_bit -1 downto 0);
            dina : in signed(data_bit -1 downto 0);
        --output--
            doutb : out signed(data_bit -1 downto 0)
        );
    end component;
    
    constant data_len : integer := data_int+data_dec;
    constant ram_addr_bit : integer := 8;
    
    signal ram_wea_a : std_logic := '0';
    signal ram_wea_b : std_logic := '0';
    signal ram_ena : std_logic := '0';
    signal store_addr : unsigned(ram_addr_bit -1 downto 0) := (others => '0');
    signal read_addr : unsigned(ram_addr_bit -1 downto 0) := (others => '0');
    signal data_out_temp_a : signed(data_len -1 downto 0) := (others => '0');
    signal data_out_temp_b : signed(data_len -1 downto 0) := (others => '0');
    signal delay_reset_layer_affine : std_logic := '0';
    signal delay_sig_36_data_out : unsigned(1 downto 0) := (others => '0');
    signal uni_data_temp_a : signed(data_len*36 -1 downto 0) := (others => '0');
    signal uni_data_temp_b : signed(data_len*36 -1 downto 0) := (others => '0');
begin
    
    --gen store addr--
    conv_store_addr : conv_store_addr_layer4_out
        port map (
        --input--
            clk => clk,
            reset => reset_layer4,
            sig_calc_out => sig_calc_out,
        --output--
            ram_wea_a => ram_wea_a,
            ram_wea_b => ram_wea_b,
            addr => store_addr
        );
    
    --gen read addr--
    conv_read_addr : conv_layer_affine_read_addr_re
        port map (
        --input--
            clk => clk,
            reset => reset_layer_affine,
            count => count_layer_affine,
        --output--
            sig_36_data_out => delay_sig_36_data_out(0),
            sig_uni_data_out => sig_uni_data_out,
            addr => read_addr
        );
        
    --store ram a--
    ram_a : store_layer4_out_ram
        generic map (
            data_bit => data_len,
            addr_bit => ram_addr_bit
        )
        port map (
        --input--
            clk => clk,
            ena => ram_ena,
            wea => ram_wea_a,
            addra => store_addr,
            addrb => read_addr,
            dina => in_data,
        --output--
            doutb => data_out_temp_a
        );
    
    --store ram b--
    ram_b : store_layer4_out_ram
        generic map (
            data_bit => data_len,
            addr_bit => ram_addr_bit
        )
        port map (
        --input--
            clk => clk,
            ena => ram_ena,
            wea => ram_wea_b,
            addra => store_addr,
            addrb => read_addr,
            dina => in_data,
        --output--
            doutb => data_out_temp_b
        );
    
    --fucntion--
    ram_ena <= not delay_reset_layer_affine;
    
    --batch data--
    process(clk)
    begin
        if (clk'event and clk='1') then
            if (reset_layer_affine = '1') then
                uni_data_temp_a <= (others => '0');
                uni_data_temp_b <= (others => '0');
            else
                uni_data_temp_a(uni_data_temp_a'length-35*data_len -1 downto 0) <= data_out_temp_a;--layer_4 0~15ch data
                uni_data_temp_a(uni_data_temp_a'length -1 downto data_len) <= uni_data_temp_a(uni_data_temp_a'length-data_len -1 downto 0);
                uni_data_temp_b(uni_data_temp_b'length-35*data_len -1 downto 0) <= data_out_temp_b;--layer_4 16~31ch data
                uni_data_temp_b(uni_data_temp_b'length -1 downto data_len) <= uni_data_temp_b(uni_data_temp_b'length-data_len -1 downto 0);
            end if;
        end if;
    end process;
    
    process(clk)
    begin
        if (clk'event and clk='1') then
            if (to_integer(count_layer_affine) = 194) then
                uni_data_a(uni_data_a'length -1 downto uni_data_a'length-data_len*12)  <= uni_data_temp_a(data_len*12 -1 downto 0);
                uni_data_b(uni_data_b'length -1 downto uni_data_b'length-data_len*12)  <= uni_data_temp_b(data_len*12 -1 downto 0);
            elsif (delay_sig_36_data_out(1) = '1') then
                uni_data_a <= uni_data_temp_a;
                uni_data_b <= uni_data_temp_b;
            else
                uni_data_a <= (others => '0');
                uni_data_b <= (others => '0');
            end if;
        end if;
    end process;
    sig_36_data_out <= delay_sig_36_data_out(1);
    
    --DFF--
    process(clk)
    begin
        if (clk'event and clk='1') then
            delay_reset_layer_affine <= reset_layer_affine;
            delay_sig_36_data_out(1) <= delay_sig_36_data_out(0);
        end if;
    end process;
end Behavioral;